@@ -1,15 +1,17 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2002-2006, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2002-2011, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: NotDefinedError.scala 5880 2006-03-02 23:05:17 +0000 (Thu, 02 Mar 2006) mihaylov $
 
 
-package scala;
+package scala
 
-
-final class NotDefinedError(msg: String) extends Error("not defined: " + msg);
+/**
+ * @since 2.0
+ */
+@deprecated("Use a custom Error class instead", "2.8.0")
+final class NotDefinedError(msg: String) extends Error("not defined: " + msg)
