@@ -1,14 +1,14 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2005-2009, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2005-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: NetKernel.scala 16881 2009-01-09 16:28:11Z cunei $
 
-package scala.actors.remote
+package scala.actors
+package remote
 
 import scala.collection.mutable.{HashMap, HashSet}
 
@@ -26,7 +26,7 @@ case class Locator(node: Node, name: Symbol)
  * @version 0.9.17
  * @author Philipp Haller
  */
-class NetKernel(service: Service) {
+private[remote] class NetKernel(service: Service) {
 
   def sendToNode(node: Node, msg: AnyRef) = {
     val bytes = service.serializer.serialize(msg)
@@ -139,7 +139,7 @@ class NetKernel(service: Service) {
 
   def terminate() {
     // tell all proxies to terminate
-    proxies.values foreach { p => p.send(Terminate, null) }
+    proxies.values foreach { _.send(Terminate, null) }
 
     // tell service to terminate
     service.terminate()
