@@ -1,36 +1,36 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2009, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: ImmutableSetAdaptor.scala 16894 2009-01-13 13:09:41Z cunei $
 
 
-package scala.collection.mutable
+package scala.collection
+package mutable
 
 
 /** This class can be used as an adaptor to create mutable sets from
- *  immutable set implementations. Only method <code>empty</code> has
+ *  immutable set implementations. Only method `empty` has
  *  to be redefined if the immutable set on which this mutable set is
- *  originally based is not empty. <code>empty</code> is supposed to
+ *  originally based is not empty. `empty` is supposed to
  *  return the representation of an empty set.
  *
  *  @author  Matthias Zenger
  *  @version 1.0, 21/07/2003
+ *  @since   1
  */
-@serializable
-class ImmutableSetAdaptor[A](protected var set: immutable.Set[A]) extends Set[A] {
+class ImmutableSetAdaptor[A](protected var set: immutable.Set[A]) extends Set[A] with Serializable {
 
-  def size: Int = set.size
+  override def size: Int = set.size
 
   override def isEmpty: Boolean = set.isEmpty
 
   def contains(elem: A): Boolean = set.contains(elem)
 
-  override def foreach(f: A => Unit): Unit = set.foreach(f)
+  override def foreach[U](f: A =>  U): Unit = set.foreach(f)
 
   override def exists(p: A => Boolean): Boolean = set.exists(p)
 
@@ -38,12 +38,16 @@ class ImmutableSetAdaptor[A](protected var set: immutable.Set[A]) extends Set[A]
 
   override def toString = set.toString
 
-  def elements: Iterator[A] = set.elements
+  def iterator: Iterator[A] = set.iterator
 
-  def +=(elem: A): Unit = { set = set + elem }
+  @deprecated("use `iterator' instead", "2.8.0")
+  override def elements: Iterator[A] = iterator
 
-  def -=(elem: A): Unit = { set = set - elem }
+  def +=(elem: A): this.type = { set = set + elem; this }
+
+  def -=(elem: A): this.type = { set = set - elem; this }
 
   override def clear(): Unit = { set = set.empty }
 
 }
+
