@@ -1,24 +1,26 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2009, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |                                         **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: QueueProxy.scala 16894 2009-01-13 13:09:41Z cunei $
 
 
-package scala.collection.mutable
+package scala.collection
+package mutable
 
-
-/** <code>Queue</code> objects implement data structures that allow to
+/** `Queue` objects implement data structures that allow to
  *  insert and retrieve elements in a first-in-first-out (FIFO) manner.
- *
+ *  
+ *  @tparam A   type of the elements in this queue proxy.
+ *  
  *  @author  Matthias Zenger
  *  @version 1.1, 03/05/2004
+ *  @since   1
  */
-trait QueueProxy[A] extends Queue[A] with SeqProxy[A] {
+trait QueueProxy[A] extends Queue[A] with Proxy {
 
   def self: Queue[A]
 
@@ -42,15 +44,7 @@ trait QueueProxy[A] extends Queue[A] with SeqProxy[A] {
    *
    *  @param  elem        the element to insert
    */
-  override def +=(elem: A): Unit = self += elem
-
-  /** Adds all elements provided by an <code>Iterable</code> object
-   *  at the end of the queue. The elements are prepended in the order they
-   *  are given out by the iterator.
-   *
-   *  @param  iter        an iterable object
-   */
-  override def ++=(iter: Iterable[A]): Unit = self ++= iter
+  override def +=(elem: A): this.type = { self += elem; this }
 
   /** Adds all elements provided by an iterator
    *  at the end of the queue. The elements are prepended in the order they
@@ -58,7 +52,10 @@ trait QueueProxy[A] extends Queue[A] with SeqProxy[A] {
    *
    *  @param  iter        an iterator
    */
-  override def ++=(it: Iterator[A]): Unit = self ++= it
+  override def ++=(it: TraversableOnce[A]): this.type = {
+    self ++= it
+    this
+  }
 
   /** Adds all elements to the queue.
    *
@@ -89,7 +86,7 @@ trait QueueProxy[A] extends Queue[A] with SeqProxy[A] {
    *
    *  @return an iterator over all queue elements.
    */
-  override def elements: Iterator[A] = self.elements
+  override def iterator: Iterator[A] = self.iterator
 
   /** This method clones the queue.
    *
