@@ -1,24 +1,27 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Subscriber.scala 6679 2006-03-10 16:09:08Z odersky $
 
 
-package scala.collection.mutable;
+package scala.collection
+package mutable
 
 
 /** <code>Subscriber[A, B]</code> objects may subscribe to events of
  *  type <code>A</code> published by an object of type <code>B</code>.
- *  <code>B</code> is typically a subtype of <code>Publisher</code>.
- * 
+ *  <code>B</code> is typically a subtype of <a href="Publisher.html"
+ *  target="contentFrame"><code>Publisher</code></a>.
+ *
  *  @author  Matthias Zenger
- *  @version 1.0, 08/07/2003
+ *  @author  Martin Odersky
+ *  @version 2.8
+ *  @since   1
  */
-trait Subscriber[-A, -B] {
-    def notify(pub: B, event: A): Unit;
+trait Subscriber[-Evt, -Pub] {
+  def notify(pub: Pub, event: Evt): Unit
 }
