@@ -1,25 +1,30 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Lock.scala 5889 2006-03-05 00:33:02Z mihaylov $
 
 
-package scala.concurrent;
-
+package scala.concurrent
 
+/** This class ...
+ *
+ *  @author  Martin Odersky
+ *  @version 1.0, 10/03/2003
+ */
 class Lock {
-  var available = true;
+  var available = true
+
   def acquire = synchronized {
-    if (!available) wait();
+    while (!available) wait()
     available = false
   }
+
   def release = synchronized {
-    available = true;
+    available = true
     notify()
   }
 }
