@@ -1,19 +1,19 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |                                         **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id:Boolean.scala 6853 2006-03-20 16:58:47 +0100 (Mon, 20 Mar 2006) dubochet $
 
 
-package scala.dbc.datatype;
+package scala.dbc
+package datatype;
 
 
 /** The SQL type for a truth value. */
-class Boolean extends DataType {
+@deprecated(DbcIsDeprecated, "2.9.0") class Boolean extends DataType {
   
   def isEquivalent (datatype:DataType) = datatype match {
     case dt:Boolean => true
