@@ -1,21 +1,21 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |                                         **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: ExactNumeric.scala 9043 2006-10-24 14:46:55Z dubochet $
 
 
-package scala.dbc.datatype;
+package scala.dbc
+package datatype;
 
 
 /** A type category for all SQL types that store constant-precision
   * numbers.
   */
-abstract class ExactNumeric[Type](
+@deprecated(DbcIsDeprecated, "2.9.0") abstract class ExactNumeric[Type](
   override val nativeTypeId: DataType.Id
 ) extends datatype.Numeric[Type](nativeTypeId) {
   
