@@ -1,20 +1,20 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |                                         **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: String.scala 5889 2006-03-05 00:33:02 +0000 (Sun, 05 Mar 2006) mihaylov $
 
 
-package scala.dbc.datatype;
+package scala.dbc
+package datatype;
 
 
 /** A type category for all SQL types that store strings of elements.
  */
-abstract class String extends DataType {
+@deprecated(DbcIsDeprecated, "2.9.0") abstract class String extends DataType {
   
   /** The maximal possible length of the string defined in characters.
    *  This is an implementation-specific value.
