@@ -1,19 +1,19 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |                                         **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Unknown.scala 9110 2006-11-01 16:03:28Z mihaylov $
 
 
-package scala.dbc.datatype;
+package scala.dbc
+package datatype;
 
 
 /** The SQL type for a truth value. */
-class Unknown extends DataType {
+@deprecated(DbcIsDeprecated, "2.9.0") class Unknown extends DataType {
   
   def isEquivalent(datatype: DataType) = datatype match {
     case dt: Unknown =>
@@ -29,6 +29,6 @@ class Unknown extends DataType {
 
   /** A SQL-99 compliant string representation of the type. */
   override def sqlString: java.lang.String =
-    error("The 'UNKNOWN' data type cannot be represented.");
+    sys.error("The 'UNKNOWN' data type cannot be represented.");
 
 }
