@@ -1,29 +1,28 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Expression.scala 5889 2006-03-05 00:33:02 +0000 (Sun, 05 Mar 2006) mihaylov $
 
 
-package scala.dbc.statement;
+package scala.dbc
+package statement
 
 
 /** An expression that calculates some value from fields. */
-abstract class Expression extends Relation {
+@deprecated(DbcIsDeprecated, "2.9.0") abstract class Expression extends Relation {
   
-  def fieldTypes: List[DataType] = Nil;
+  def fieldTypes: List[DataType] = Nil
   
   /** A SQL-99 compliant string representation of the expression. */
-  def sqlString: String = {
-    "SELECT " + sqlInnerString
-  }
+  def sqlString: String = "SELECT " + sqlInnerString
   
   /** A SQL-99 compliant string representation of the relation sub-
-   * statement. This only has a meaning inside another statement. */
-  def sqlInnerString: String;
+   *  statement. This only has a meaning inside another statement.
+   */
+  def sqlInnerString: String
   
 }
