@@ -1,24 +1,24 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2007, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id:SetQuantifier.scala 6853 2006-03-20 16:58:47 +0100 (Mon, 20 Mar 2006) dubochet $
 
 
-package scala.dbc.statement
+package scala.dbc
+package statement
 
 
 /** A set quantifier that defines the collection type of a relation. */
-abstract class SetQuantifier {
+@deprecated(DbcIsDeprecated, "2.9.0") abstract class SetQuantifier {
   /** A SQL-99 compliant string representation of the set quantifier. */
   def sqlString: String
 }
 
-object SetQuantifier {
+@deprecated(DbcIsDeprecated, "2.9.0") object SetQuantifier {
 
   /** A set quantifier that defines a relation as being a bag. That means
    *  that duplicates are allowed.
