@@ -1,18 +1,19 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |                                         **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Aggregate.scala 5889 2006-03-05 00:33:02Z mihaylov $
 
 
-package scala.dbc.statement.expression;
+package scala.dbc
+package statement
+package expression
 
 
-abstract class Aggregate extends Expression {
+@deprecated(DbcIsDeprecated, "2.9.0") abstract class Aggregate extends Expression {
   
   def aggregateName: String;
   
