@@ -1,23 +1,24 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |                                         **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id:SetFunction.scala 6853 2006-03-20 16:58:47 +0100 (Mon, 20 Mar 2006) dubochet $
 
 
-package scala.dbc.statement.expression;
+package scala.dbc
+package statement
+package expression;
 
 
-abstract class SetFunction {
+@deprecated(DbcIsDeprecated, "2.9.0") abstract class SetFunction {
   /** A SQL-99 compliant string representation of the set quantifier. */
   def sqlString: String;
 }
 
-object SetFunction {
+@deprecated(DbcIsDeprecated, "2.9.0") object SetFunction {
   abstract class Asterisk extends SetFunction {
     def sqlString = "(*)";
   }
@@ -34,6 +35,6 @@ object SetFunction {
     );
   }
   abstract class Binary extends SetFunction {
-    def sqlString = error("Binary set function is not supported yet.");
+    def sqlString = sys.error("Binary set function is not supported yet.");
   }
 }
