@@ -1,18 +1,18 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |                                         **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: ApproximateNumeric.scala 6656 2006-03-09 15:36:04 +0000 (Thu, 09 Mar 2006) emir $
 
 
-package scala.dbc.value;
+package scala.dbc
+package value;
 
 
-abstract class ApproximateNumeric [Type] extends Value {
+@deprecated(DbcIsDeprecated, "2.9.0") abstract class ApproximateNumeric [Type] extends Value {
   
   val dataType: datatype.ApproximateNumeric[Type];
   
@@ -20,7 +20,7 @@ abstract class ApproximateNumeric [Type] extends Value {
   
   }
 
-object ApproximateNumeric {
+@deprecated(DbcIsDeprecated, "2.9.0") object ApproximateNumeric {
   
   implicit def approximateNumericToFloar (obj:value.ApproximateNumeric[Float]): Float = obj.nativeValue;
   implicit def approximateNumericToDouble (obj:value.ApproximateNumeric[Double]): Double = obj.nativeValue;
