@@ -1,18 +1,20 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2002-2007, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2002-2011, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: remote.scala 9400 2006-11-28 17:22:45 +0000 (Tue, 28 Nov 2006) michelou $
-
-
 package scala
 
+import annotation.target._
+
 /**
  * An annotation that designates the definition to which it is applied as deprecated.
  * Access to the member then generates a deprecated warning.
+ *
+ * @since 2.3
  */
-class deprecated extends StaticAnnotation {}
+@getter @setter @beanGetter @beanSetter
+class deprecated(message: String = "", since: String = "") extends annotation.StaticAnnotation
