@@ -1,12 +1,11 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2002-2008, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2002-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: BeanDisplayName.scala 14416 2008-03-19 01:17:25Z mihaylov $
 
 package scala.reflect
 
@@ -15,5 +14,5 @@ package scala.reflect
  *
  *  @author Ross Judson (rjudson@managedobjects.com)
  */
-class BeanDisplayName(val name: String) extends Annotation
+class BeanDisplayName(val name: String) extends annotation.Annotation
 
