@@ -1,24 +1,23 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2002-2007, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2002-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Symbol.scala 11127 2007-05-21 14:56:06Z spoon $
 
 
 package scala.reflect
 
-import Predef._
-
+/** This type is required by the compiler and <b>should not be used in client code</b>. */
 abstract class Symbol {
   val owner: Symbol
   val name: String
   val tpe: Type
 }
 
+/** This type is required by the compiler and <b>should not be used in client code</b>. */
 abstract class GlobalSymbol(val fullname: String) extends Symbol {
   private val pointIndex = fullname.lastIndexOf(".")
   val owner: Symbol =
@@ -29,34 +28,43 @@ abstract class GlobalSymbol(val fullname: String) extends Symbol {
     else fullname.substring(pointIndex+1, fullname.length())
 }
 
+/** This type is required by the compiler and <b>should not be used in client code</b>. */
 abstract class LocalSymbol extends Symbol
 
+/** This type is required by the compiler and <b>should not be used in client code</b>. */
 case class Class(override val fullname: String) extends GlobalSymbol(fullname) {
   val tpe = NamedType(fullname)
 }
 
+/** This type is required by the compiler and <b>should not be used in client code</b>. */
 case class Method(override val fullname: String, tpe: Type) extends GlobalSymbol(fullname)
 
+/** This type is required by the compiler and <b>should not be used in client code</b>. */
 case class Field(override val fullname: String, tpe: Type) extends GlobalSymbol(fullname)
 
+/** This type is required by the compiler and <b>should not be used in client code</b>. */
 case class TypeField(override val fullname: String, tpe: Type) extends GlobalSymbol(fullname)
 
-case class LocalValue(owner: Symbol, name: String, tpe: Type) extends LocalSymbol
+/** This type is required by the compiler and <b>should not be used in client code</b>. */case class LocalValue(owner: Symbol, name: String, tpe: Type) extends LocalSymbol
 
+/** This type is required by the compiler and <b>should not be used in client code</b>. */
 case class LocalMethod(owner: Symbol, name: String, tpe: Type) extends LocalSymbol
 
+/** This type is required by the compiler and <b>should not be used in client code</b>. */
 case object NoSymbol extends Symbol {
   val owner = null
   val name = null
   val tpe = NoType
 }
 
+/** This type is required by the compiler and <b>should not be used in client code</b>. */
 case object RootSymbol extends Symbol {
   val owner = NoSymbol
   val name = "<root>"
   val tpe = NoPrefix
 }
 
+/** This type is required by the compiler and <b>should not be used in client code</b>. */
 case class LabelSymbol(val name: String) extends Symbol {
   val owner = NoSymbol
   val tpe = NamedType("scala.Unit")
