@@ -1,18 +1,13 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2002-2007, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2002-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Type.scala 14706 2008-04-18 15:59:23Z dubochet $
-
-
 package scala.reflect
 
-import Predef._
-
 /** This type is required by the compiler and <b>should not be used in client code</b>. */
 abstract class Type
 
@@ -48,19 +43,14 @@ case class TypeBounds(lo: Type, hi: Type) extends Type
 
 /** This type is required by the compiler and <b>should not be used in client code</b>.
   * <code>(formals1 ... formalsn) restpe</code> */
-case class MethodType(formals: List[Type], restpe: Type) extends Type
+case class MethodType(formals: List[Symbol], restpe: Type) extends Type
 
+/** This type is required by the compiler and <b>should not be used in client code</b>. */
+case class NullaryMethodType(resultType: Type) extends Type
 
 /** This type is required by the compiler and <b>should not be used in client code</b>. */
 case class PolyType(typeParams: List[Symbol], typeBounds: List[(Type, Type)], resultType: Type) extends Type
 
-
-/** This type is required by the compiler and <b>should not be used in client code</b>. */
-class ImplicitMethodType(formals: List[Type], restpe: Type)
-extends MethodType(formals, restpe)
-
-
-
 /* Standard pattern match:
 
   case reflect.NoPrefix =>
@@ -71,6 +61,7 @@ extends MethodType(formals, restpe)
   case reflect.ThisType(clazz) => 
   case reflect.AppliedType(tpe, args) => 
   case reflect.TypeBounds(lo, hi) => 
-  case reflect.MethodType(formals, restpe) => //can also be ImplicitMethodType
+  case reflect.MethodType(formals, restpe) =>
+  case reflect.NullaryMethodType(restpe) =>
   case reflect.PolyType(typeParams, typeBounds, resultType) =>
 */
