@@ -1,35 +1,19 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2002-2007, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2002-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: RichFloat.scala 14532 2008-04-07 12:23:22Z washburn $
-
-
 package scala.runtime
 
-
-import Predef._
-
-final class RichFloat(x: Float) extends Proxy with Ordered[Float] {
-
-  // Proxy.self
-  def self: Any = x
-
-  // Ordered[Float].compare
-  //def compare(y: Float): Int = if (x < y) -1 else if (x > y) 1 else 0
-  def compare(y: Float): Int = java.lang.Float.compare(x, y)
-
-  def min(y: Float) = Math.min(x, y)
-  def max(y: Float) = Math.max(x, y)
-  def abs: Float = Math.abs(x)
-
-  def round: Int = Math.round(x)
-  def ceil: Float = Math.ceil(x).toFloat
-  def floor: Float = Math.floor(x).toFloat
+final class RichFloat(val self: Float) extends FractionalProxy[Float] {
+  protected val integralNum = Numeric.FloatAsIfIntegral
+  
+  def round: Int   = math.round(self)
+  def ceil: Float  = math.ceil(self).toFloat
+  def floor: Float = math.floor(self).toFloat
 
   /** Converts an angle measured in degrees to an approximately equivalent
    *  angle measured in radians.
@@ -37,7 +21,7 @@ final class RichFloat(x: Float) extends Proxy with Ordered[Float] {
    *  @param  x an angle, in degrees
    *  @return the measurement of the angle <code>x</code> in radians.
    */
-  def toRadians: Float = Math.toRadians(x).toFloat
+  def toRadians: Float = math.toRadians(self).toFloat
 
   /** Converts an angle measured in radians to an approximately equivalent
    *  angle measured in degrees.
@@ -45,12 +29,11 @@ final class RichFloat(x: Float) extends Proxy with Ordered[Float] {
    *  @param  x angle, in radians
    *  @return the measurement of the angle <code>x</code> in degrees.
    */
-  def toDegrees: Float = Math.toDegrees(x).toFloat
+  def toDegrees: Float = math.toDegrees(self).toFloat
 
   // isNaN is provided by the implicit conversion to java.lang.Float
-  // def isNaN: Boolean = java.lang.Float.isNaN(x)
-  def isInfinity: Boolean = java.lang.Float.isInfinite(x)
-  def isPosInfinity: Boolean = isInfinity && x > 0.0f
-  def isNegInfinity: Boolean = isInfinity && x < 0.0f
-
+  // def isNaN: Boolean = java.lang.Float.isNaN(self)
+  def isInfinity: Boolean = java.lang.Float.isInfinite(self)
+  def isPosInfinity: Boolean = isInfinity && self > 0.0f
+  def isNegInfinity: Boolean = isInfinity && self < 0.0f
 }
