@@ -9,7 +9,7 @@ package ast.parser
 import javac._
 
 /** An nsc sub-component.
- */ 
+ */
 abstract class SyntaxAnalyzer extends SubComponent with Parsers with MarkupParsers with Scanners with JavaParsers with JavaScanners {
 
   val phaseName = "parser"
@@ -21,12 +21,15 @@ abstract class SyntaxAnalyzer extends SubComponent with Parsers with MarkupParse
     override val keepsTypeParams = false
 
     def apply(unit: global.CompilationUnit) {
-      global.informProgress("parsing " + unit)
-      unit.body =     
+      import global._
+      informProgress("parsing " + unit)
+      unit.body =
         if (unit.source.file.name.endsWith(".java")) new JavaUnitParser(unit).parse()
-        else if (!global.reporter.incompleteHandled) new UnitParser(unit).smartParse()        
-        else new UnitParser(unit).parse()
-      if (global.settings.Yrangepos.value && !global.reporter.hasErrors) global.validatePositions(unit.body)
+        else if (reporter.incompleteHandled) new UnitParser(unit).parse()
+        else new UnitParser(unit).smartParse()
+
+      if (settings.Yrangepos.value && !reporter.hasErrors)
+        validatePositions(unit.body)
     }
   }
 }
