@@ -9,20 +9,20 @@ package settings
 /** Taking flag checking to a somewhat higher level. */
 trait AestheticSettings {
   def settings: Settings
-  
+
   // Some(value) if setting has been set by user, None otherwise.
   def optSetting[T](s: Settings#Setting): Option[T] =
     if (s.isDefault) None else Some(s.value.asInstanceOf[T])
-  
+
   def script       = optSetting[String](settings.script)
   def encoding     = optSetting[String](settings.encoding)
   def sourceReader = optSetting[String](settings.sourceReader)
-  
+
   def debug           = settings.debug.value
   def declsOnly       = false
   def deprecation     = settings.deprecation.value
   def experimental    = settings.Xexperimental.value
-  def fatalWarnings   = settings.Xwarnfatal.value
+  def fatalWarnings   = settings.fatalWarnings.value
   def logClasspath    = settings.Ylogcp.value
   def printStats      = settings.Ystatistics.value
   def richExes        = settings.YrichExes.value || sys.props.traceSourcePath.isSet
