@@ -15,12 +15,12 @@ import scala.io.Source
 
 /** A mutable Settings object.
  */
-class MutableSettings(val errorFn: String => Unit) extends AbsSettings with ScalaSettings with Mutable {  
+class MutableSettings(val errorFn: String => Unit) extends AbsSettings with ScalaSettings with Mutable {
   type ResultOfTryToSet = List[String]
 
   /** Iterates over the arguments applying them to settings where applicable.
    *  Then verifies setting dependencies are met.
-   * 
+   *
    *  This temporarily takes a boolean indicating whether to keep
    *  processing if an argument is seen which is not a command line option.
    *  This is an expedience for the moment so that you can say
@@ -28,10 +28,10 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
    *    scalac -d /tmp foo.scala -optimise
    *
    *  while also allowing
-   * 
+   *
    *    scala Program opt opt
    *
-   *  to get their arguments. 
+   *  to get their arguments.
    *
    *  Returns (success, List of unprocessed arguments)
    */
@@ -49,6 +49,13 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
             errorFn("bad option: '" + x + "'")
             (false, args)
           }
+          // discard empties, sometimes they appear because of ant or etc.
+          // but discard carefully, because an empty string is valid as an argument
+          // to an option, e.g. -cp "" .  So we discard them only when they appear
+          // in option position.
+          else if (x == "") {
+            loop(xs, residualArgs)
+          }
           else lookupSetting(x) match {
             case Some(s) if s.shouldStopProcessing  => (checkDependencies, newArgs)
             case _                                  => loop(newArgs, residualArgs)
@@ -59,7 +66,7 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
           else (checkDependencies, args)
         }
     }
-    loop(arguments filterNot (_ == ""), Nil)
+    loop(arguments, Nil)
   }
   def processArgumentString(params: String) = processArguments(splitParams(params), true)
 
@@ -79,7 +86,7 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
    *  '*' in this list.
    */
   lazy val outputDirs = new OutputDirs
-  
+
   /** A list of settings which act based on prefix rather than an exact
    *  match.  This is basically -D and -J.
    */
@@ -151,14 +158,14 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
         }
     }
   }
-  
+
   /** Initializes these settings for embedded use by type `T`.
   * The class loader defining `T` should provide resources `app.class.path`
   * and `boot.class.path`.  These resources should contain the application
   * and boot classpaths in the same form as would be passed on the command line.*/
   def embeddedDefaults[T: Manifest]: Unit =
     embeddedDefaults(implicitly[Manifest[T]].erasure.getClassLoader)
-    
+
   /** Initializes these settings for embedded use by a class from the given class loader.
   * The class loader for `T` should provide resources `app.class.path`
   * and `boot.class.path`.  These resources should contain the application
@@ -168,10 +175,10 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
     getClasspath("app", loader) foreach { classpath.value = _ }
     getClasspath("boot", loader) foreach { bootclasspath append _ }
   }
-  
+
   /** The parent loader to use for the interpreter.*/
   private[nsc] var explicitParentLoader: Option[ClassLoader] = None
-  
+
   /** Retrieves the contents of resource "${id}.class.path" from `loader`
   * (wrapped in Some) or None if the resource does not exist.*/
   private def getClasspath(id: String, loader: ClassLoader): Option[String] =
@@ -206,7 +213,7 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
     protected var v: T
     protected var setByUser: Boolean = false
     def postSetHook(): Unit
-    
+
     def isDefault: Boolean = !setByUser
     def value: T = v
     def value_=(arg: T) = {
@@ -290,23 +297,23 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
           }
       }
     }
-    
+
     /** Return the source file path(s) which correspond to the given
      *  classfile path and SourceFile attribute value, subject to the
      *  condition that source files are arranged in the filesystem
      *  according to Java package layout conventions.
-     *  
+     *
      *  The given classfile path must be contained in at least one of
      *  the specified output directories. If it does not then this
      *  method returns Nil.
-     *  
+     *
      *  Note that the source file is not required to exist, so assuming
      *  a valid classfile path this method will always return a list
      *  containing at least one element.
-     *  
+     *
      *  Also that if two or more source path elements target the same
      *  output directory there will be two or more candidate source file
-     *  paths. 
+     *  paths.
      */
     def srcFilesFor(classFile : AbstractFile, srcPath : String) : List[AbstractFile] = {
       def isBelow(srcDir: AbstractFile, outDir: AbstractFile) =
@@ -340,7 +347,7 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
     private var _helpSyntax = name
     override def helpSyntax: String = _helpSyntax
     def withHelpSyntax(s: String): this.type    = { _helpSyntax = s ; this }
-    
+
     /** Abbreviations for this setting */
     private var _abbreviations: List[String] = Nil
     override def abbreviations = _abbreviations
@@ -350,7 +357,7 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
     private var dependency: Option[(Setting, String)] = None
     override def dependencies = dependency.toList
     def dependsOn(s: Setting, value: String): this.type = { dependency = Some((s, value)); this }
-    
+
     private var _deprecationMessage: Option[String] = None
     override def deprecationMessage = _deprecationMessage
     def withDeprecationMessage(msg: String): this.type = { _deprecationMessage = Some(msg) ; this }
@@ -412,6 +419,8 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
     def unparse: List[String] =
       if (value == default) Nil
       else List(name, value.toString)
+
+    withHelpSyntax(name + " <n>")
   }
 
   /** A setting represented by a boolean flag (false, unless set) */
@@ -428,7 +437,7 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
       value = s.equalsIgnoreCase("true")
     }
   }
-  
+
   /** A special setting for accumulating arguments like -Dfoo=bar. */
   class PrefixSetting private[nsc](
     name: String,
@@ -437,7 +446,7 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
   extends Setting(name, descr) {
     type T = List[String]
     protected var v: List[String] = Nil
-  
+
     def tryToSet(args: List[String]) = args match {
       case x :: xs if x startsWith prefix =>
         v = v :+ x
@@ -467,7 +476,7 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
 
     withHelpSyntax(name + " <" + arg + ">")
   }
-  
+
   class PathSetting private[nsc](
     name: String,
     descr: String,
@@ -478,7 +487,7 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
     import util.ClassPath.join
     def prepend(s: String) = prependPath.value = join(s, prependPath.value)
     def append(s: String) = appendPath.value = join(appendPath.value, s)
-    
+
     override def value = join(
       prependPath.value,
       super.value,
@@ -517,14 +526,14 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
       Some(rest)
     }
     override def tryToSetColon(args: List[String]) = tryToSet(args)
-    override def tryToSetFromPropertyValue(s: String) = tryToSet(s.trim.split(" +").toList)
+    override def tryToSetFromPropertyValue(s: String) = tryToSet(s.trim.split(',').toList)
     def unparse: List[String] = value map { name + ":" + _ }
 
     withHelpSyntax(name + ":<" + arg + ">")
   }
 
-  /** A setting represented by a string in a given set of <code>choices</code>,
-   *  (<code>default</code> unless set).
+  /** A setting represented by a string in a given set of `choices`,
+   *  (`default` unless set).
    */
   class ChoiceSetting private[nsc](
     name: String,
@@ -558,7 +567,7 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
 
   /** A setting represented by a list of strings which should be prefixes of
    *  phase names. This is not checked here, however.  Alternatively the string
-   *  "all" can be used to represent all phases.
+   *  `"all"` can be used to represent all phases.
    *  (the empty list, unless set)
    */
   class PhasesSetting private[nsc](
@@ -570,7 +579,7 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
     override def value = if (v contains "all") List("all") else super.value
     private lazy val (numericValues, stringValues) =
       value filterNot (_ == "" ) partition (_ forall (ch => ch.isDigit || ch == '-'))
-    
+
     /** A little ad-hoc parsing.  If a string is not the name of a phase, it can also be:
      *    a phase id: 5
      *    a phase id range: 5-10 (inclusive of both ends)
@@ -591,7 +600,7 @@ class MutableSettings(val errorFn: String => Unit) extends AbsSettings with Scal
         case Nil    => _ => false
         case fns    => fns.reduceLeft((f1, f2) => id => f1(id) || f2(id))
       }
-    
+
     def tryToSet(args: List[String]) = errorAndValue("missing phase", None)
     override def tryToSetColon(args: List[String]) = args match {
       case Nil  => errorAndValue("missing phase", None)
