@@ -11,7 +11,7 @@ import scala.collection.{ mutable, immutable }
 import symtab._
 import Flags._
 
-abstract class Erasure extends AddInterfaces 
+abstract class Erasure extends AddInterfaces
                           with typechecker.Analyzer
                           with TypingTransformers
                           with ast.TreeDSL
@@ -31,16 +31,16 @@ abstract class Erasure extends AddInterfaces
 
   /** An extractor object for generic arrays */
   object GenericArray {
-    
+
     /** Is `tp` an unbounded generic type (i.e. which could be instantiated
-     *  with primitive as well as class types)?. 
+     *  with primitive as well as class types)?.
      */
     private def genericCore(tp: Type): Type = tp.normalize match {
       case TypeRef(_, sym, _) if sym.isAbstractType && !sym.owner.isJavaDefined =>
         tp
-      case ExistentialType(tparams, restp) => 
+      case ExistentialType(tparams, restp) =>
         genericCore(restp)
-      case _ => 
+      case _ =>
         NoType
     }
 
@@ -51,12 +51,12 @@ abstract class Erasure extends AddInterfaces
     def unapply(tp: Type): Option[(Int, Type)] = tp.normalize match {
       case TypeRef(_, ArrayClass, List(arg)) =>
         genericCore(arg) match {
-          case NoType => 
+          case NoType =>
             unapply(arg) match {
               case Some((level, core)) => Some((level + 1, core))
               case None => None
             }
-          case core => 
+          case core =>
             Some(1, core)
         }
       case ExistentialType(tparams, restp) =>
@@ -66,6 +66,39 @@ abstract class Erasure extends AddInterfaces
     }
   }
 
+  // A type function from T => Class[U], used to determine the return
+  // type of getClass calls.  The returned type is:
+  //
+  //  1. If T is a value type, Class[T].
+  //  2. If T is anonymous or a refinement type, calculate the intersection
+  //     dominator of the parents T', and Class[_ <: T'].
+  //  3. If T is a phantom type (Any or AnyVal), Class[_].
+  //  4. Otherwise, Class[_ <: T].
+  //
+  // Note: AnyVal cannot be Class[_ <: AnyVal] because if the static type of the
+  // receiver is AnyVal, it implies the receiver is boxed, so the correct
+  // class object is that of java.lang.Integer, not Int.
+  //
+  // TODO: If T is final, return type could be Class[T].  Should it?
+  def getClassReturnType(tp: Type): Type = {
+    def mkClass(targs: List[Type]) = typeRef(ClassClass.tpe.prefix, ClassClass, targs)
+    val tparams = ClassClass.typeParams
+    val sym     = tp.typeSymbol
+
+    if (tparams.isEmpty) mkClass(Nil)   // call must be coming post-erasure
+    else if (isValueClass(sym)) mkClass(List(tp.widen))
+    else if (sym.isLocalClass) getClassReturnType(erasure.intersectionDominator(tp.parents))
+    else {
+      val eparams    = typeParamsToExistentials(ClassClass, tparams)
+      val upperBound = if (isPhantomClass(sym)) AnyClass.tpe else tp.widen
+
+      existentialAbstraction(
+        eparams,
+        mkClass(List(eparams.head setInfo TypeBounds.upper(upperBound) tpe))
+      )
+    }
+  }
+
   private def unboundedGenericArrayLevel(tp: Type): Int = tp match {
     case GenericArray(level, core) if !(core <:< AnyRefClass.tpe) => level
     case _ => 0
@@ -152,7 +185,7 @@ abstract class Erasure extends AddInterfaces
           MethodType(
             cloneSymbols(params) map (p => p.setInfo(apply(p.tpe))),
             if (restpe.typeSymbol == UnitClass)
-              erasedTypeRef(UnitClass) 
+              erasedTypeRef(UnitClass)
             else if (settings.YdepMethTpes.value)
               // this replaces each typeref that refers to an argument by the type `p.tpe` of the actual argument p (p in params)
               apply(mt.resultType(params map (_.tpe)))
@@ -175,8 +208,8 @@ abstract class Erasure extends AddInterfaces
   }
 
   private object NeedsSigCollector extends TypeCollector(false) {
-    def traverse(tp: Type) { 
-      if (!result) { 
+    def traverse(tp: Type) {
+      if (!result) {
         tp match {
           case st: SubType =>
             traverse(st.supertype)
@@ -200,7 +233,7 @@ abstract class Erasure extends AddInterfaces
     }
   }
 
-  private def needsJavaSig(tp: Type) = !settings.Ynogenericsig.value && NeedsSigCollector.collect(tp)
+  def needsJavaSig(tp: Type) = !settings.Ynogenericsig.value && NeedsSigCollector.collect(tp)
 
   // only refer to type params that will actually make it into the sig, this excludes:
   // * higher-order type parameters
@@ -230,9 +263,9 @@ abstract class Erasure extends AddInterfaces
     }
   }
   // for debugging signatures: traces logic given system property
-  private val traceProp = sys.BooleanProp keyExists "scalac.sigs.trace"
+  private val traceProp = (sys.BooleanProp keyExists "scalac.sigs.trace").value // performance: get the value here
   private val traceSig  = util.Tracer(traceProp)
-  
+
   /** This object is only used for sanity testing when -check:genjvm is set.
    *  In that case we make sure that the erasure of the `normalized' type
    *  is the same as the erased type that's generated. Normalization means
@@ -248,7 +281,7 @@ abstract class Erasure extends AddInterfaces
         val tpe1 = squashBoxed(tpe)
         if (tpe1 eq tpe) t
         else ExistentialType(tparams, tpe1)
-      case t =>  
+      case t =>
         if (boxedClass contains t.typeSymbol) ObjectClass.tpe
         else tp
     }
@@ -267,9 +300,9 @@ abstract class Erasure extends AddInterfaces
         if (sym == ArrayClass && args.nonEmpty)
           if (unboundedGenericArrayLevel(tp1) == 1) ObjectClass.tpe
           else mapOver(tp1)
-        else if (sym == AnyClass || sym == AnyValClass || sym == SingletonClass) 
+        else if (sym == AnyClass || sym == AnyValClass || sym == SingletonClass)
           ObjectClass.tpe
-        else if (sym == UnitClass) 
+        else if (sym == UnitClass)
           BoxedUnitClass.tpe
         else if (sym == NothingClass)
           RuntimeNothingClass.tpe
@@ -311,9 +344,9 @@ abstract class Erasure extends AddInterfaces
       case RefinedType(parents, _) => parents map normalize
       case tp                      => tp :: Nil
     }
-    
+
     def jsig(tp0: Type, existentiallyBound: List[Symbol] = Nil, toplevel: Boolean = false, primitiveOK: Boolean = true): String = {
-      val tp = tp0.dealias 
+      val tp = tp0.dealias
       tp match {
         case st: SubType =>
           jsig(st.supertype, existentiallyBound, toplevel, primitiveOK)
@@ -329,9 +362,9 @@ abstract class Erasure extends AddInterfaces
             } else {
               boxedSig(tp)
             }
-          def classSig: String = 
+          def classSig: String =
             "L"+atPhase(currentRun.icodePhase)(sym.fullName + global.genJVM.moduleSuffix(sym)).replace('.', '/')
-          def classSigSuffix: String = 
+          def classSigSuffix: String =
             "."+sym.name
 
           // If args isEmpty, Array is being used as a higher-kinded type
@@ -343,9 +376,9 @@ abstract class Erasure extends AddInterfaces
             assert(!sym.isAliasType, "Unexpected alias type: " + sym)
             TVAR_TAG.toString+sym.name+";"
           }
-          else if (sym == AnyClass || sym == AnyValClass || sym == SingletonClass) 
+          else if (sym == AnyClass || sym == AnyValClass || sym == SingletonClass)
             jsig(ObjectClass.tpe)
-          else if (sym == UnitClass) 
+          else if (sym == UnitClass)
             jsig(BoxedUnitClass.tpe)
           else if (sym == NothingClass)
             jsig(RuntimeNothingClass.tpe)
@@ -381,7 +414,7 @@ abstract class Erasure extends AddInterfaces
           assert(tparams.nonEmpty)
           def boundSig(bounds: List[Type]) = {
             val (isTrait, isClass) = bounds partition (_.typeSymbol.isTrait)
-            
+
             ":" + (
               if (isClass.isEmpty) "" else boxedSig(isClass.head)
             ) + (
@@ -389,7 +422,7 @@ abstract class Erasure extends AddInterfaces
             )
           }
           def paramSig(tsym: Symbol) = tsym.name + boundSig(hiBounds(tsym.info.bounds))
-          
+
           val paramString = if (toplevel) tparams map paramSig mkString ("<", "", ">") else ""
           traceSig("PolyType", (tparams, restpe))(paramString + jsig(restpe))
         case MethodType(params, restpe) =>
@@ -428,7 +461,7 @@ abstract class Erasure extends AddInterfaces
   /** Remove duplicate references to class Object in a list of parent classes */
   private def removeDoubleObject(tps: List[Type]): List[Type] = tps match {
     case List() => List()
-    case tp :: tps1 => 
+    case tp :: tps1 =>
       if (tp.typeSymbol == ObjectClass) tp :: tps1.filter(_.typeSymbol != ObjectClass)
       else tp :: removeDoubleObject(tps1)
   }
@@ -444,9 +477,9 @@ abstract class Erasure extends AddInterfaces
   def transformInfo(sym: Symbol, tp: Type): Type = {
     if (sym == Object_asInstanceOf)
       sym.info
-    else if (sym == Object_isInstanceOf || sym == ArrayClass) 
+    else if (sym == Object_isInstanceOf || sym == ArrayClass)
       PolyType(sym.info.typeParams, erasure(sym.info.resultType))
-    else if (sym.isAbstractType) 
+    else if (sym.isAbstractType)
       TypeBounds(WildcardType, WildcardType)
     else if (sym.isTerm && sym.owner == ArrayClass) {
       if (sym.isClassConstructor)
@@ -455,7 +488,7 @@ abstract class Erasure extends AddInterfaces
             MethodType(cloneSymbols(params) map (p => p.setInfo(erasure(p.tpe))),
                        typeRef(erasure(pre), sym, args))
         }
-      else if (sym.name == nme.apply) 
+      else if (sym.name == nme.apply)
         tp
       else if (sym.name == nme.update)
         (tp: @unchecked) match {
@@ -474,10 +507,10 @@ abstract class Erasure extends AddInterfaces
       tp
     } else {
 /*
-      val erased = 
-        if (sym.isGetter && sym.tpe.isInstanceOf[MethodType]) 
+      val erased =
+        if (sym.isGetter && sym.tpe.isInstanceOf[MethodType])
           erasure mapOver sym.tpe // for getters, unlike for normal methods, always convert Unit to BoxedUnit.
-        else 
+        else
           erasure(tp)
 */
       transformMixinInfo(erasure(tp))
@@ -491,6 +524,12 @@ abstract class Erasure extends AddInterfaces
       case _ => tp.deconst
     }
   }
+  // Methods on Any/Object which we rewrite here while we still know what
+  // is a primitive and what arrived boxed.
+  private lazy val interceptedMethods = Set[Symbol](Any_##, Object_##, Any_getClass) ++ (
+    // Each value class has its own getClass for ultra-precise class object typing.
+    ScalaValueClasses map (_.tpe member nme.getClass_)
+  )
 
 // -------- erasure on trees ------------------------------------------
 
@@ -515,10 +554,10 @@ abstract class Erasure extends AddInterfaces
    */
 
   /** The modifier typer which retypes with erased types. */
-  class Eraser(context: Context) extends Typer(context) {  
-    private def safeToRemoveUnbox(cls: Symbol): Boolean = 
+  class Eraser(context: Context) extends Typer(context) {
+    private def safeToRemoveUnbox(cls: Symbol): Boolean =
       (cls == definitions.NullClass) || isBoxedValueClass(cls)
-      
+
     /** Box `tree' of unboxed type */
     private def box(tree: Tree): Tree = tree match {
       case LabelDef(name, params, rhs) =>
@@ -529,12 +568,13 @@ abstract class Erasure extends AddInterfaces
           case UnitClass  =>
             if (treeInfo isPureExpr tree) REF(BoxedUnit_UNIT)
             else BLOCK(tree, REF(BoxedUnit_UNIT))
+          case NothingClass => tree // a non-terminating expression doesn't need boxing
           case x          =>
             assert(x != ArrayClass)
             tree match {
               /** Can't always remove a Box(Unbox(x)) combination because the process of boxing x
-               *  may lead to throwing an exception. 
-               *  
+               *  may lead to throwing an exception.
+               *
                *  This is important for specialization: calls to the super constructor should not box/unbox specialized
                *  fields (see TupleX). (ID)
                */
@@ -576,8 +616,14 @@ abstract class Erasure extends AddInterfaces
     /** Generate a synthetic cast operation from tree.tpe to pt.
      *  @pre pt eq pt.normalize
      */
-    private def cast(tree: Tree, pt: Type): Tree =
-      tree AS_ATTR pt
+    private def cast(tree: Tree, pt: Type): Tree = {
+      if (pt.typeSymbol == UnitClass) {
+        // See SI-4731 for one example of how this occurs.
+        log("Attempted to cast to Unit: " + tree)
+        tree.duplicate setType pt
+      }
+      else tree AS_ATTR pt
+    }
 
     private def isUnboxedValueMember(sym: Symbol) =
       sym != NoSymbol && isValueClass(sym.owner)
@@ -596,22 +642,22 @@ abstract class Erasure extends AddInterfaces
       else if (isValueClass(tree.tpe.typeSymbol) && !isValueClass(pt.typeSymbol))
         adaptToType(box(tree), pt)
       else if (tree.tpe.isInstanceOf[MethodType] && tree.tpe.params.isEmpty) {
-        if (!tree.symbol.isStable) assert(false, "adapt "+tree+":"+tree.tpe+" to "+pt)
+        assert(tree.symbol.isStable, "adapt "+tree+":"+tree.tpe+" to "+pt)
         adaptToType(Apply(tree, List()) setPos tree.pos setType tree.tpe.resultType, pt)
-      } else if (pt <:< tree.tpe) 
+      } else if (pt <:< tree.tpe)
         cast(tree, pt)
       else if (isValueClass(pt.typeSymbol) && !isValueClass(tree.tpe.typeSymbol))
         adaptToType(unbox(tree, pt), pt)
-      else 
+      else
         cast(tree, pt)
     }
-    
+
     // @PP 1/25/2011: This is less inaccurate than it was (I removed
     // BoxedAnyArray, asInstanceOf$erased, and other long ago eliminated symbols)
     // but I do not think it yet describes the code beneath it.
 
     /**  Replace member references as follows:
-     * 
+     *
      *   - `x == y` for == in class Any becomes `x equals y` with equals in class Object.
      *   - `x != y` for != in class Any becomes `!(x equals y)` with equals in class Object.
      *   - x.asInstanceOf[T] becomes x.$asInstanceOf[T]
@@ -633,7 +679,7 @@ abstract class Erasure extends AddInterfaces
           if (isNumericValueClass(qualClass) && isNumericValueClass(targClass))
             // convert numeric type casts
             atPos(tree.pos)(Apply(Select(qual1, "to" + targClass.name), List()))
-          else 
+          else
 */
           if (isValueClass(targClass)) unbox(qual1, targ.tpe)
           else tree
@@ -652,7 +698,7 @@ abstract class Erasure extends AddInterfaces
               qual1 = box(qual1)
             else if (!isValueClass(qual1.tpe.typeSymbol) && isUnboxedValueMember(tree.symbol))
               qual1 = unbox(qual1, tree.symbol.owner.tpe)
-            
+
             if (isValueClass(tree.symbol.owner) && !isValueClass(qual1.tpe.typeSymbol))
               tree.symbol = NoSymbol
             else if (qual1.tpe.isInstanceOf[MethodType] && qual1.tpe.params.isEmpty) {
@@ -667,7 +713,7 @@ abstract class Erasure extends AddInterfaces
         case SelectFromArray(qual, name, erasure) =>
           var qual1 = typedQualifier(qual)
           if (!(qual1.tpe <:< erasure)) qual1 = cast(qual1, erasure)
-          Select(qual1, name) copyAttrs tree 
+          Select(qual1, name) copyAttrs tree
         case _ =>
           tree
       }
@@ -681,7 +727,7 @@ abstract class Erasure extends AddInterfaces
     /** A replacement for the standard typer's `typed1' method.
      */
     override protected def typed1(tree: Tree, mode: Int, pt: Type): Tree = {
-      val tree1 = try { 
+      val tree1 = try {
         super.typed1(adaptMember(tree), mode, pt)
       } catch {
         case er: TypeError =>
@@ -690,7 +736,7 @@ abstract class Erasure extends AddInterfaces
           er.printStackTrace
           abort()
         case ex: Exception =>
-          //if (settings.debug.value) 
+          //if (settings.debug.value)
           Console.println("exception when typing " + tree);
           throw ex
       }
@@ -711,7 +757,7 @@ abstract class Erasure extends AddInterfaces
         case Ident(_) | Select(_, _) =>
           if (tree1.symbol.isOverloaded) {
             val first = tree1.symbol.alternatives.head
-            val sym1 = tree1.symbol.filter { 
+            val sym1 = tree1.symbol.filter {
               alt => alt == first || !(first.tpe looselyMatches alt.tpe)
             }
             if (tree.symbol ne sym1) {
@@ -729,7 +775,7 @@ abstract class Erasure extends AddInterfaces
   /** The erasure transformer */
   class ErasureTransformer(unit: CompilationUnit) extends Transformer {
     /** Emit an error if there is a double definition. This can happen if:
-     * 
+     *
      *  - A template defines two members with the same name and erased type.
      *  - A template defines and inherits two members `m` with different types,
      *    but their erased types are the same.
@@ -772,9 +818,9 @@ abstract class Erasure extends AddInterfaces
 
       val opc = new overridingPairs.Cursor(root) {
         override def exclude(sym: Symbol): Boolean =
-          (!sym.isTerm || sym.isPrivate || super.exclude(sym) 
+          (!sym.isTerm || sym.isPrivate || super.exclude(sym)
            // specialized members have no type history before 'specialize', causing double def errors for curried defs
-           || !sym.hasTypeAt(currentRun.refchecksPhase.id)) 
+           || !sym.hasTypeAt(currentRun.refchecksPhase.id))
 
         override def matches(sym1: Symbol, sym2: Symbol): Boolean =
           atPhase(phase.next)(sym1.tpe =:= sym2.tpe)
@@ -798,9 +844,9 @@ abstract class Erasure extends AddInterfaces
       for (bc <- root.info.baseClasses.tail; other <- bc.info.decls.toList) {
         if (other.isTerm && !other.isConstructor && !(other hasFlag (PRIVATE | BRIDGE))) {
           for (member <- root.info.nonPrivateMember(other.name).alternatives) {
-            if (member != other && 
-                !(member hasFlag BRIDGE) && 
-                atPhase(phase.next)(member.tpe =:= other.tpe) && 
+            if (member != other &&
+                !(member hasFlag BRIDGE) &&
+                atPhase(phase.next)(member.tpe =:= other.tpe) &&
                 !atPhase(refchecksPhase.next)(
                   root.thisType.memberType(member) matches root.thisType.memberType(other))) {
               if (settings.debug.value) log("" + member.locationString + " " + member.infosString + other.locationString + " " + other.infosString);
@@ -812,18 +858,18 @@ abstract class Erasure extends AddInterfaces
 */
 
     /**  Add bridge definitions to a template. This means:
-     *  
+     *
      *   If there is a concrete member `m` which overrides a member in a base
      *   class of the template, and the erased types of the two members differ,
      *   and the two members are not inherited or defined by some parent class
      *   of the template, then a bridge from the overridden member `m1` to the
      *   member `m0` is added. The bridge has the erased type of `m1` and
      *   forwards to `m0`.
-     *  
+     *
      *   No bridge is added if there is already a bridge to `m0` with the erased
      *   type of `m1` in the template.
      */
-    private def bridgeDefs(owner: Symbol): (List[Tree], immutable.Set[Symbol]) = {  
+    private def bridgeDefs(owner: Symbol): (List[Tree], immutable.Set[Symbol]) = {
       var toBeRemoved: immutable.Set[Symbol] = immutable.Set()
       //println("computing bridges for " + owner)//DEBUG
       assert(phase == currentRun.erasurePhase)
@@ -831,8 +877,8 @@ abstract class Erasure extends AddInterfaces
       val bridgesScope = new Scope
       val bridgeTarget = new mutable.HashMap[Symbol, Symbol]
       var bridges: List[Tree] = List()
-      val opc = atPhase(currentRun.explicitouterPhase) {     
-        new overridingPairs.Cursor(owner) {    
+      val opc = atPhase(currentRun.explicitouterPhase) {
+        new overridingPairs.Cursor(owner) {
           override def parents: List[Type] = List(owner.info.parents.head)
           override def exclude(sym: Symbol): Boolean =
             !sym.isMethod || sym.isPrivate || super.exclude(sym)
@@ -895,9 +941,9 @@ abstract class Erasure extends AddInterfaces
         if (other.isMethod && !other.isConstructor) {
           for (member <- site.nonPrivateMember(other.name).alternatives) {
             if (member != other &&
-                !(member hasFlag DEFERRED) && 
+                !(member hasFlag DEFERRED) &&
                 (site.memberType(member) matches site.memberType(other)) &&
-                !(site.parents exists (p => 
+                !(site.parents exists (p =>
                   (p.symbol isSubClass member.owner) && (p.symbol isSubClass other.owner)))) {
 ...
              }
@@ -908,13 +954,13 @@ abstract class Erasure extends AddInterfaces
       if (base.isTrait) stats
       else {
         val (bridges, toBeRemoved) = bridgeDefs(base)
-        if (bridges.isEmpty) stats 
+        if (bridges.isEmpty) stats
         else (stats filterNot (stat => toBeRemoved contains stat.symbol)) ::: bridges
       }
 
     /**  Transform tree at phase erasure before retyping it.
      *   This entails the following:
-     *  
+     *
      *   - Remove all type parameters in class and method definitions.
      *   - Remove all abstract and alias type definitions.
      *   - Remove all type applications other than those involving a type test or cast.
@@ -942,26 +988,26 @@ abstract class Erasure extends AddInterfaces
               if ((fun.symbol == Any_isInstanceOf || fun.symbol == Object_isInstanceOf) &&
                   unboundedGenericArrayLevel(arg.tpe) > 0) =>
           val level = unboundedGenericArrayLevel(arg.tpe)
-          def isArrayTest(arg: Tree) = 
+          def isArrayTest(arg: Tree) =
             gen.mkRuntimeCall("isArray", List(arg, Literal(Constant(level))))
           global.typer.typedPos(tree.pos) {
             if (level == 1) isArrayTest(qual)
             else
               gen.evalOnce(qual, currentOwner, unit) { qual1 =>
                 gen.mkAnd(
-                  Apply(TypeApply(Select(qual1(), fun.symbol), 
-                                  List(TypeTree(erasure(arg.tpe)))), 
+                  Apply(TypeApply(Select(qual1(), fun.symbol),
+                                  List(TypeTree(erasure(arg.tpe)))),
                         List()),
                   isArrayTest(qual1()))
               }
           }
-        case TypeApply(fun, args) if (fun.symbol.owner != AnyClass && 
+        case TypeApply(fun, args) if (fun.symbol.owner != AnyClass &&
                                       fun.symbol != Object_asInstanceOf &&
                                       fun.symbol != Object_isInstanceOf) =>
           // leave all other type tests/type casts, remove all other type applications
           preErase(fun)
-        case Apply(fn @ Select(qual, name), args) if (fn.symbol.owner == ArrayClass) => 
-          if (unboundedGenericArrayLevel(qual.tpe.widen) == 1) 
+        case Apply(fn @ Select(qual, name), args) if (fn.symbol.owner == ArrayClass) =>
+          if (unboundedGenericArrayLevel(qual.tpe.widen) == 1)
             // convert calls to apply/update/length on generic arrays to
             // calls of ScalaRunTime.array_xxx method calls
             global.typer.typedPos(tree.pos) { gen.mkRuntimeCall("array_"+name, qual :: args) }
@@ -973,17 +1019,24 @@ abstract class Erasure extends AddInterfaces
               SelectFromArray(qual, name, erasure(qual.tpe)).copyAttrs(fn),
               args)
 
-        case Apply(fn @ Select(qual, _), Nil) if fn.symbol == Any_## || fn.symbol == Object_## =>
-          // This is unattractive, but without it we crash here on ().## because after
-          // erasure the ScalaRunTime.hash overload goes from Unit => Int to BoxedUnit => Int.
-          // This must be because some earlier transformation is being skipped on ##, but so
-          // far I don't know what.  For null we now define null.## == 0.
-          val arg = qual.tpe.typeSymbolDirect match {
-            case UnitClass  => BLOCK(qual, REF(BoxedUnit_UNIT))   // ({ expr; UNIT }).##
-            case NullClass  => LIT(0)                             // (null: Object).##
-            case _          => qual
+        case Apply(fn @ Select(qual, _), Nil) if interceptedMethods(fn.symbol) =>
+          if (fn.symbol == Any_## || fn.symbol == Object_##) {
+            // This is unattractive, but without it we crash here on ().## because after
+            // erasure the ScalaRunTime.hash overload goes from Unit => Int to BoxedUnit => Int.
+            // This must be because some earlier transformation is being skipped on ##, but so
+            // far I don't know what.  For null we now define null.## == 0.
+            val arg = qual.tpe.typeSymbolDirect match {
+              case UnitClass  => BLOCK(qual, REF(BoxedUnit_UNIT))   // ({ expr; UNIT }).##
+              case NullClass  => LIT(0)                             // (null: Object).##
+              case _          => qual
+            }
+            Apply(gen.mkAttributedRef(scalaRuntimeHash), List(arg))
           }
-          Apply(gen.mkAttributedRef(scalaRuntimeHash), List(arg))
+          // Rewrite 5.getClass to ScalaRunTime.anyValClass(5)
+          else if (isValueClass(qual.tpe.typeSymbol))
+            Apply(gen.mkAttributedRef(scalaRuntimeAnyValClass), List(qual))
+          else
+            tree
 
         case Apply(fn, args) =>
           if (fn.symbol == Any_asInstanceOf)
@@ -991,14 +1044,14 @@ abstract class Erasure extends AddInterfaces
               case TypeApply(Select(qual, _), List(targ)) =>
                 if (qual.tpe <:< targ.tpe) {
                   atPos(tree.pos) { Typed(qual, TypeTree(targ.tpe)) }
-                } else if (isNumericValueClass(qual.tpe.typeSymbol) && 
+                } else if (isNumericValueClass(qual.tpe.typeSymbol) &&
                            isNumericValueClass(targ.tpe.typeSymbol)) {
                   // convert numeric type casts
                   val cname = newTermName("to" + targ.tpe.typeSymbol.name)
                   val csym = qual.tpe.member(cname)
                   assert(csym != NoSymbol)
                   atPos(tree.pos) { Apply(Select(qual, csym), List()) }
-                } else 
+                } else
                   tree
             }
             // todo: also handle the case where the singleton type is buried in a compound
@@ -1007,11 +1060,11 @@ abstract class Erasure extends AddInterfaces
               case TypeApply(sel @ Select(qual, name), List(targ)) =>
                 if (qual.tpe != null && isValueClass(qual.tpe.typeSymbol) && targ.tpe != null && targ.tpe <:< AnyRefClass.tpe)
                   unit.error(sel.pos, "isInstanceOf cannot test if value types are references.")
-              
+
                 def mkIsInstanceOf(q: () => Tree)(tp: Type): Tree =
                   Apply(
                     TypeApply(
-                      Select(q(), Object_isInstanceOf) setPos sel.pos, 
+                      Select(q(), Object_isInstanceOf) setPos sel.pos,
                       List(TypeTree(tp) setPos targ.pos)) setPos fn.pos,
                     List()) setPos tree.pos
                 targ.tpe match {
@@ -1021,9 +1074,18 @@ abstract class Erasure extends AddInterfaces
                       Apply(Select(qual, cmpOp), List(gen.mkAttributedQualifier(targ.tpe)))
                     }
                   case RefinedType(parents, decls) if (parents.length >= 2) =>
-                    gen.evalOnce(qual, currentOwner, unit) { q =>
+                    // Optimization: don't generate isInstanceOf tests if the static type
+                    // conforms, because it always succeeds.  (Or at least it had better.)
+                    // At this writing the pattern matcher generates some instance tests
+                    // involving intersections where at least one parent is statically known true.
+                    // That needs fixing, but filtering the parents here adds an additional
+                    // level of robustness (in addition to the short term fix.)
+                    val parentTests = parents filterNot (qual.tpe <:< _)
+
+                    if (parentTests.isEmpty) Literal(Constant(true))
+                    else gen.evalOnce(qual, currentOwner, unit) { q =>
                       atPos(tree.pos) {
-                        parents map mkIsInstanceOf(q) reduceRight gen.mkAnd
+                        parentTests map mkIsInstanceOf(q) reduceRight gen.mkAnd
                       }
                     }
                   case _ =>
@@ -1056,7 +1118,7 @@ abstract class Erasure extends AddInterfaces
           }
           def isAccessible(sym: Symbol) = localTyper.context.isAccessible(sym, sym.owner.thisType)
           if (!isAccessible(owner) && qual.tpe != null) {
-            // Todo: Figure out how qual.tpe could be null in the check above (it does appear in build where SwingWorker.this 
+            // Todo: Figure out how qual.tpe could be null in the check above (it does appear in build where SwingWorker.this
             // has a null type).
             val qualSym = qual.tpe.widen.typeSymbol
             if (isAccessible(qualSym) && !qualSym.isPackageClass && !qualSym.isPackageObjectClass) {
@@ -1075,7 +1137,7 @@ abstract class Erasure extends AddInterfaces
         case Match(selector, cases) =>
           Match(Typed(selector, TypeTree(selector.tpe)), cases)
 
-        case Literal(ct) if ct.tag == ClassTag 
+        case Literal(ct) if ct.tag == ClassTag
                          && ct.typeValue.typeSymbol != definitions.UnitClass =>
           treeCopy.Literal(tree, Constant(erasure(ct.typeValue)))
 
@@ -1089,7 +1151,7 @@ abstract class Erasure extends AddInterfaces
         //   value array_this is not a member of object scala.runtime.ScalaRunTime
         //
         // What the heck is array_this? See preTransformer in this file:
-        //   gen.mkRuntimeCall("array_"+name, qual :: args) 
+        //   gen.mkRuntimeCall("array_"+name, qual :: args)
         if (tree.symbol == ArrayClass && !tree.isType) tree
         else {
           val tree1 = preErase(tree)
