@@ -1,12 +1,11 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2009, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Benchmark.scala 16894 2009-01-13 13:09:41Z cunei $
 
 
 package scala.testing
@@ -39,7 +38,13 @@ import compat.Platform
  */
 trait Benchmark {
 
-  /** this method should be implemented by the concrete benchmark */
+  /** this method should be implemented by the concrete benchmark.
+   *  This method is called by the benchmarking code for a number of times.
+   *  The GC is called before each call to 'run'.
+   *
+   *  @see setUp
+   *  @see tearDown
+   */
   def run()
 
   var multiplier = 1
@@ -53,44 +58,65 @@ trait Benchmark {
    */
   def runBenchmark(noTimes: Int): List[Long] =
     for (i <- List.range(1, noTimes + 1)) yield {
+      setUp
       val startTime = Platform.currentTime
       var i = 0; while (i < multiplier) {
         run()
         i += 1
       }
       val stopTime = Platform.currentTime
+      tearDown
       Platform.collectGarbage
 
       stopTime - startTime
     }
 
+  /** Prepare any data needed by the benchmark, but whose execution time
+   *  should not be measured. This method is run before each call to the
+   *  benchmark payload, 'run'.
+   */
+  def setUp {
+  }
+
+  /** Perform cleanup operations after each 'run'. For micro benchmarks,
+   *  think about using the result of 'run' in a way that prevents the JVM
+   *  to dead-code eliminate the whole 'run' method. For instance, print or
+   *  write the results to a file. The execution time of this method is not
+   *  measured.
+   */
+  def tearDown {
+  }
+
   /** a string that is written at the beginning of the output line
    *   that contains the timings. By default, this is the class name.
    */
   def prefix: String = getClass().getName()
 
   /**
-   * The entry point. It takes two arguments (n), (name) 
+   * The entry point. It takes two arguments (n),
    *  and an optional argument multiplier (mult).
-   *  (n) is the number of consecutive runs, (name) the name 
-   *  of a log file where to append the times.
-   *  if (mult) is present, the same thing is repeated (mult)
+   *  (n) is the number of consecutive runs,
+   *  if (mult) is present, the n runs are repeated (mult)
    *  times.
    */
   def main(args: Array[String]) {
-    if (args.length > 1) {
-      val logFile = new java.io.FileWriter(args(1), true) // append, not overwrite
-      if (args.length >= 3)
-         multiplier = args(2).toInt
+    if (args.length > 0) {
+      val logFile = new java.io.OutputStreamWriter(System.out)
+      if (args.length > 1) multiplier = args(1).toInt
       logFile.write(prefix)
       for (t <- runBenchmark(args(0).toInt))
-        logFile.write("\t\t" + t)
+        logFile.write("\t" + t)
 
       logFile.write(Platform.EOL)
       logFile.flush()
     } else {
-      Console.println("Usage: scala benchmarks.program <runs> <logfile>")
-      Console.println("   or: scala benchmarks.program <runs> <logfile> <multiplier>")
+      println("Usage: scala benchmarks.program <runs> ")
+      println("   or: scala benchmarks.program <runs> <multiplier>")
+      println("""
+    The benchmark is run <runs> times, forcing a garbage collection between runs. The optional
+    <multiplier> causes the benchmark to be repeated <multiplier> times, each time for <runs>
+    executions.
+      """)
     }
   }
 }
