@@ -1,12 +1,11 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2006-2009, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2006-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: CharArrayReader.scala 16894 2009-01-13 13:09:41Z cunei $
 
 package scala.util.parsing.input
 
@@ -16,9 +15,6 @@ package scala.util.parsing.input
  */
 object CharArrayReader {
   final val EofCh = '\032'
-
-  /** @deprecated  This should probably be LF instead? */
-  @deprecated final val CR = '\015'
 }
 
 /** A character array reader reads a stream of characters (keeping track of their positions) 
