@@ -1,81 +1,59 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2009, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Null.scala 16894 2009-01-13 13:09:41Z cunei $
 
 
 package scala.xml
 
-case object Null extends MetaData {
-
-  /** appends given MetaData items to this MetaData list */
-  override def append(m: MetaData): MetaData = m
-
-  override def containedIn1(m: MetaData): Boolean = false
-  
-  /** returns its argument */
-  def copy(next: MetaData) = next
-
-  override def elements = Iterator.empty
+import Utility.{ isNameStart }
 
+/** Essentially, every method in here is a dummy, returning Zero[T].
+ *  It provides a backstop for the unusual collection defined by MetaData,
+ *  sort of a linked list of tails.
+ */
+case object Null extends MetaData {    
+  override def iterator = Iterator.empty
+  override def append(m: MetaData, scope: NamespaceBinding = TopScope): MetaData = m
   override def filter(f: MetaData => Boolean): MetaData = this
 
-  /** returns null */
+  def copy(next: MetaData) = next
   def getNamespace(owner: Node) = null
 
-  final override def hasNext = false
-
-  final override def length = 0
-
-  final override def length(i: Int) = i
-
+  override def hasNext = false
+  def next = null
+  def key = null
+  def value = null
   def isPrefixed = false
 
-  /** deep equals method */
-  override def equals(that: Any) = that match {
-    case m: MetaData => m.length == 0
-    case _ => false
+  override def length = 0
+  override def length(i: Int) = i
+  
+  override def strict_==(other: Equality) = other match {
+    case x: MetaData  => x.length == 0
+    case _            => false
   }
+  override def basisForHashCode: Seq[Any] = Nil
 
-  def equals1(that:MetaData) = that.length == 0
-
-  def key = null
-
-  def value = null
-
-  override def map(f: MetaData => Text): List[Text] = Nil
-
-  def next = null
-
-  /** null */
+  def apply(namespace: String, scope: NamespaceBinding, key: String) = null
   def apply(key: String) = {
-    if(!Parsing.isNameStart (key charAt 0))
+    if (!isNameStart(key.head))
       throw new IllegalArgumentException("not a valid attribute name '"+key+"', so can never match !")
+      
     null
   }
 
-  /** gets value of qualified (prefixed) attribute with given key */
-  def apply(namespace: String, scope: NamespaceBinding, key: String) = null
-
-  override def hashCode(): Int = 0
-
+  def toString1(sb: StringBuilder) = ()
   override def toString1(): String = ""
-
-  //appends string representations of single attribute to StringBuilder
-  def toString1(sb:StringBuilder) = {}
-
   override def toString(): String = ""
 
-  override def toString(sb: StringBuilder): StringBuilder = sb
-
+  override def buildString(sb: StringBuilder): StringBuilder = sb
   override def wellformed(scope: NamespaceBinding) = true
 
   def remove(key: String) = this
-
   def remove(namespace: String, scope: NamespaceBinding, key: String) = this
 }
