@@ -1,16 +1,14 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2002-2009, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2002-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Scanner.scala 16857 2009-01-07 20:34:03Z cunei $
-
-
-package scala.xml.dtd
 
+package scala.xml
+package dtd
 
 /** Scanner for regexps (content models in DTD element declarations) 
  *  todo: cleanup
@@ -28,7 +26,7 @@ class Scanner extends Tokens with parsing.TokenTests {
   /** initializes the scanner on input s */
   final def initScanner(s: String) {
     value = ""
-    it = (s).elements
+    it = (s).iterator
     token = 1+END
     next
     nextToken
@@ -46,13 +44,10 @@ class Scanner extends Tokens with parsing.TokenTests {
   final def next = if (it.hasNext) c = it.next else c = ENDCH
 
   final def acc(d: Char) { 
-    if (c == d) next else error("expected '"+d+"' found '"+c+"' !");
+    if (c == d) next else sys.error("expected '"+d+"' found '"+c+"' !");
   }
 
-  final def accS(ds: Seq[Char]) { 
-    val jt = ds.elements
-    while (jt.hasNext) { acc(jt.next) }
-  }
+  final def accS(ds: Seq[Char]) { ds foreach acc }
 
   final def readToken: Int = 
     if (isSpace(c)) {
@@ -70,7 +65,7 @@ class Scanner extends Tokens with parsing.TokenTests {
       case ENDCH => END
       case _     => 
         if (isNameStart(c)) name; // NAME 
-        else { error("unexpected character:"+c); END }
+        else sys.error("unexpected character:" + c)
     }
   
   final def name = {
