@@ -1,44 +1,34 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2002-2009, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2002-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |                                         **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: LoggedNodeFactory.scala 16894 2009-01-13 13:09:41Z cunei $
+package scala.xml
+package factory
 
+/** This class logs what the nodefactory is actually doing.
+ *  If you want to see what happens during loading, use it like this:
+{{{
+object testLogged extends Application {
+  val x = new scala.xml.parsing.NoBindingFactoryAdapter
+        with scala.xml.factory.LoggedNodeFactory[scala.xml.Elem]
+        with scala.util.logging.ConsoleLogger
 
-package scala.xml.factory
-
-
-/** <p>
- *    This class logs what the nodefactory is actually doing.
- *    If you want to see what happens during loading, use it like this:
- *  </p><pre>
- *  <b>object</b> testLogged <b>extends</b> Application {
- *
- *    <b>val</b> x = <b>new</b> scala.xml.nobinding.NoBindingFactoryAdapter 
- *          <b>with</b> scala.xml.LoggedNodeFactory[scala.xml.Elem]()
- *          <b>with</b> scala.util.logging.ConsoleLogger;
- *
- *    Console.println("Start");
- *
- *    <b>val</b> doc = x.loadXML(new org.xml.sax.InputSource("http://lamp.epfl.ch/~buraq"));
- *
- *    Console.println("End");
- *
- *    Console.println(doc);
- *  }</pre>
+  Console.println("Start")
+  val doc = x.load(new java.net.URL("http://lampsvn.epfl.ch/svn-repos/scala/scala/trunk/build.xml"))
+  Console.println("End")
+  Console.println(doc)
+}
+}}}
  *
  *  @author  Burak Emir
  *  @version 1.0
  */
-abstract class LoggedNodeFactory[A <: Node] 
-extends NodeFactory[A]
-with scala.util.logging.Logged {
-
-  // configuration values;
+trait LoggedNodeFactory[A <: Node] extends NodeFactory[A] with scala.util.logging.Logged {
+  // configuration values
   val logNode      = true
   val logText      = false
   val logComment   = false
@@ -58,7 +48,7 @@ with scala.util.logging.Logged {
     if (logNode)
       log("[makeNode for "+label+"]");
 
-    val hash = Utility.hashCode(pre, label, attrSeq.hashCode(), scope.hashCode(), children)
+    val hash = Utility.hashCode(pre, label, attrSeq.##, scope.##, children)
 
     /*
     if(logCompressLevel >= FULL) {
