@@ -1,23 +1,23 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2002-2009, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2002-2011, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: ConstructingHandler.scala 16894 2009-01-13 13:09:41Z cunei $
 
 
-package scala.xml.parsing
+package scala.xml
+package parsing
 
 /** Implementation of MarkupHandler that constructs nodes.
  *
  *  @author  Burak Emir
  *  @version 1.0
  */
-abstract class ConstructingHandler extends MarkupHandler {
-
+abstract class ConstructingHandler extends MarkupHandler
+{
   val preserveWS: Boolean
 
   def elem(pos: Int, pre: String, label: String, attrs: MetaData,
@@ -27,13 +27,7 @@ abstract class ConstructingHandler extends MarkupHandler {
   def procInstr(pos: Int, target: String, txt: String) =
     ProcInstr(target, txt)
 
-  def comment(pos: Int, txt: String) =
-    Comment(txt)
-
-  def entityRef(pos: Int, n: String) =
-    EntityRef(n)
-
-  def text(pos: Int, txt:String) =
-    Text(txt)
-
+  def comment(pos: Int, txt: String)  = Comment(txt)
+  def entityRef(pos: Int, n: String)  = EntityRef(n)
+  def text(pos: Int, txt: String)     = Text(txt)
 }
