@@ -1,19 +1,17 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2011, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: MarkupParser.scala 9235 2006-11-13 14:59:18Z mihaylov $
+package scala.xml
+package parsing
 
-
-package scala.xml.parsing
-
-import compat.StringBuilder
 import scala.io.Source
 import scala.xml.dtd._
+import Utility.Escapes.{ pairs => unescape }
 
 /**
  * An XML parser.
@@ -29,7 +27,20 @@ import scala.xml.dtd._
  * @author  Burak Emir
  * @version 1.0
  */
-trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef with TokenTests {
+trait MarkupParser extends MarkupParserCommon with TokenTests
+{ 
+  self: MarkupParser with MarkupHandler =>
+  
+  type PositionType = Int
+  type InputType    = Source
+  type ElementType  = NodeSeq
+  type AttributesType = (MetaData, NamespaceBinding)
+  type NamespaceType = NamespaceBinding
+
+  def truncatedError(msg: String): Nothing = throw FatalError(msg)
+  def errorNoEnd(tag: String) = throw FatalError("expected closing tag of " + tag)
+  
+  def xHandleError(that: Char, msg: String) = reportSyntaxError(msg)
 
   val input: Source
 
@@ -44,6 +55,31 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
 
   var curInput: Source = input
 
+  // See ticket #3720 for motivations.
+  private class WithLookAhead(underlying: Source) extends Source {
+    private val queue = collection.mutable.Queue[Char]()
+    def lookahead(): BufferedIterator[Char] = {
+      val iter = queue.iterator ++ new Iterator[Char] {
+        def hasNext = underlying.hasNext
+        def next() = { val x = underlying.next(); queue += x; x }
+      }
+      iter.buffered
+    }
+    val iter = new Iterator[Char] {
+      def hasNext = underlying.hasNext || !queue.isEmpty
+      def next() = if (!queue.isEmpty) queue.dequeue() else underlying.next()
+    }
+  }
+
+  def lookahead(): BufferedIterator[Char] = curInput match {
+    case curInputWLA:WithLookAhead => curInputWLA.lookahead()
+    case _ => 
+      val newInput = new WithLookAhead(curInput)
+      curInput = newInput
+      newInput.lookahead()
+  }
+ 
+
   /** the handler of the markup, returns this */
   private val handle: MarkupHandler = this
 
@@ -53,10 +89,9 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
   /** holds the position in the source file */
   var pos: Int = _
 
-
   /* used when reading external subset */
   var extIndex = -1
-
+  
   /** holds temporary values of pos */
   var tmppos: Int = _
 
@@ -81,92 +116,71 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
   def xmlProcInstr(): MetaData = {
     xToken("xml")
     xSpace
-    val Pair(md,scp) = xAttributes(TopScope)
+    val (md,scp) = xAttributes(TopScope)
     if (scp != TopScope)
       reportSyntaxError("no xmlns definitions here, please.");
     xToken('?')
     xToken('>')
     md
   }
-
-  /** &lt;? prolog ::= xml S 
-   *  // this is a bit more lenient than necessary...
+  
+  /** Factored out common code.
    */
-  def prolog(): Tuple3[Option[String], Option[String], Option[Boolean]] = {
-
-    //Console.println("(DEBUG) prolog")
-    var n = 0
+  private def prologOrTextDecl(isProlog: Boolean): (Option[String], Option[String], Option[Boolean]) = {
     var info_ver: Option[String] = None
     var info_enc: Option[String] = None
     var info_stdl: Option[Boolean] = None
 
-    var m = xmlProcInstr()
-
-    xSpace
-
-    m("version") match {
-      case null  => ;
-      case Text("1.0") => info_ver = Some("1.0"); n = n + 1
-      case _     => reportSyntaxError("cannot deal with versions != 1.0")
-    }
-
-    m("encoding") match {
-      case null => ;
-      case Text(enc) =>
-        if (!isValidIANAEncoding(enc.toString()))
-          reportSyntaxError("\"" + enc + "\" is not a valid encoding")
-        else {
-          info_enc = Some(enc.toString())
-          n = n + 1
-        }
-    }
-    m("standalone") match {
-      case null => ;
-      case Text("yes") => info_stdl = Some(true);  n = n + 1
-      case Text("no")  => info_stdl = Some(false); n = n + 1
-      case _     => reportSyntaxError("either 'yes' or 'no' expected")
-    }
-
-    if (m.length - n != 0) {
-      reportSyntaxError("VersionInfo EncodingDecl? SDDecl? or '?>' expected!");
-    }
-    //Console.println("[MarkupParser::prolog] finished parsing prolog!");
-    Tuple3(info_ver,info_enc,info_stdl)
-  }
-
-  /** prolog, but without standalone */
-  def textDecl(): Tuple2[Option[String],Option[String]] = {
-
-    var info_ver: Option[String] = None
-    var info_enc: Option[String] = None
-
     var m = xmlProcInstr()
     var n = 0
+    
+    if (isProlog)
+      xSpaceOpt
 
     m("version") match {
       case null => ;
-      case Text("1.0") => info_ver = Some("1.0"); n = n + 1
+      case Text("1.0") => info_ver = Some("1.0"); n += 1
       case _     => reportSyntaxError("cannot deal with versions != 1.0")
     }
 
     m("encoding") match {
       case null => ;
       case Text(enc)  =>
-        if (!isValidIANAEncoding(enc.toString()))
+        if (!isValidIANAEncoding(enc))
           reportSyntaxError("\"" + enc + "\" is not a valid encoding")
         else {
-          info_enc = Some(enc.toString())
-          n = n + 1
+          info_enc = Some(enc)
+          n += 1
         }
     }
+    
+    if (isProlog) {
+      m("standalone") match {
+        case null => ;
+        case Text("yes") => info_stdl = Some(true);  n += 1
+        case Text("no")  => info_stdl = Some(false); n += 1
+        case _     => reportSyntaxError("either 'yes' or 'no' expected")
+      }
+    }
 
     if (m.length - n != 0) {
-      reportSyntaxError("VersionInfo EncodingDecl? or '?>' expected!");
+      val s = if (isProlog) "SDDecl? " else ""
+      reportSyntaxError("VersionInfo EncodingDecl? %sor '?>' expected!" format s)
     }
-    //Console.println("[MarkupParser::textDecl] finished parsing textdecl");
-    Tuple2(info_ver, info_enc);
+    
+    (info_ver, info_enc, info_stdl)
   }
 
+  /** &lt;? prolog ::= xml S?
+   *  // this is a bit more lenient than necessary...
+   */
+  def prolog(): (Option[String], Option[String], Option[Boolean]) =
+    prologOrTextDecl(true)
+
+  /** prolog, but without standalone */
+  def textDecl(): (Option[String], Option[String]) =
+    prologOrTextDecl(false) match { case (x1, x2, _)  => (x1, x2) }
+
   /**
    *[22]        prolog     ::=          XMLDecl? Misc* (doctypedecl Misc*)?
    *[23]        XMLDecl    ::=          '&lt;?xml' VersionInfo EncodingDecl? SDDecl? S? '?>'
@@ -177,8 +191,6 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
    */
 
   def document(): Document = {
-
-    //Console.println("(DEBUG) document")
     doc = new Document()
 
     this.dtd = null
@@ -191,7 +203,6 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
     nextch // is prolog ?
     var children: NodeSeq = null
     if ('?' == ch) {
-      //Console.println("[MarkupParser::document] starts with xml declaration");
       nextch;
       info_prolog = prolog()
       doc.version    = info_prolog._1
@@ -199,10 +210,8 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
       doc.standAlone = info_prolog._3
 
       children = content(TopScope) // DTD handled as side effect
-    } else {
-      //Console.println("[MarkupParser::document] does not start with xml declaration");
- //
-
+    }
+    else {
       val ts = new NodeBuffer();
       content1(TopScope, ts); // DTD handled as side effect
       ts &+ content(TopScope);
@@ -211,13 +220,13 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
     //Console.println("[MarkupParser::document] children now: "+children.toList);
     var elemCount = 0;
     var theNode: Node = null;
-    for (val c <- children) c match {
+    for (c <- children) c match {
       case _:ProcInstr => ;
       case _:Comment => ;
       case _:EntityRef => // todo: fix entities, shouldn't be "special"
-        reportSyntaxError("no entity references alllowed here");
+        reportSyntaxError("no entity references allowed here");
       case s:SpecialNode =>
-        if(s.toString().trim().length() > 0) //non-empty text nodes not allowed
+        if (s.toString().trim().length > 0) //non-empty text nodes not allowed
           elemCount = elemCount + 2;
       case m:Node =>
         elemCount = elemCount + 1;
@@ -235,11 +244,25 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
 
   /** append Unicode character to name buffer*/
   protected def putChar(c: Char) = cbuf.append(c)
+  
+  /** As the current code requires you to call nextch once manually
+   *  after construction, this method formalizes that suboptimal reality.
+   */
+  def initialize: this.type = {
+    nextch
+    this
+  }
 
-  //var xEmbeddedBlock = false;
+  def ch_returning_nextch = { val res = ch ; nextch ; res }
+  def mkProcInstr(position: Int, name: String, text: String): NodeSeq =
+    handle.procInstr(position, name, text)
 
+  def mkAttributes(name: String, pscope: NamespaceBinding) =
+    if (isNameStart (ch)) xAttributes(pscope)
+    else (Null, pscope)
+  
   /** this method assign the next character to ch and advances in input */
-  def nextch: Unit = {
+  def nextch = {
     if (curInput.hasNext) {
       ch = curInput.next
       pos = curInput.pos
@@ -254,31 +277,13 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
         ch = 0.asInstanceOf[Char]
       }
     }
-  }
-
-  //final val enableEmbeddedExpressions: Boolean = false;
-
-  /** munch expected XML token, report syntax error for unexpected
-  */
-  def xToken(that: Char): Unit = {
-    if (ch == that)
-      nextch
-    else  {
-      reportSyntaxError("'" + that + "' expected instead of '" + ch + "'")
-      error("FATAL")
-    }
-  }
-
-  def xToken(that: Seq[Char]): Unit = {
-    val it = that.elements;
-    while (it.hasNext)
-      xToken(it.next);
+    ch
   }
 
   /** parse attribute and create namespace scope, metadata
    *  [41] Attributes    ::= { S Name Eq AttValue }
    */
-  def xAttributes(pscope:NamespaceBinding): Pair[MetaData,NamespaceBinding] = {
+  def xAttributes(pscope:NamespaceBinding): (MetaData,NamespaceBinding) = {
     var scope: NamespaceBinding = pscope
     var aMap: MetaData = Null
     while (isNameStart(ch)) {
@@ -290,18 +295,18 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
 
       Utility.prefix(qname) match {
         case Some("xmlns") =>
-          val prefix = qname.substring(6 /*xmlns:*/ , qname.length());
+          val prefix = qname.substring(6 /*xmlns:*/ , qname.length);
           scope = new NamespaceBinding(prefix, value, scope);
         
         case Some(prefix)       => 
-          val key = qname.substring(prefix.length()+1, qname.length());
-          aMap = new PrefixedAttribute(prefix, key, value, aMap);
+          val key = qname.substring(prefix.length+1, qname.length);
+          aMap = new PrefixedAttribute(prefix, key, Text(value), aMap);
 
         case _             => 
           if( qname == "xmlns" ) 
             scope = new NamespaceBinding(null, value, scope);
           else 
-            aMap = new UnprefixedAttribute(qname, value, aMap);
+            aMap = new UnprefixedAttribute(qname, Text(value), aMap);
       }
             
       if ((ch != '/') && (ch != '>') && ('?' != ch))
@@ -311,28 +316,7 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
     if(!aMap.wellformed(scope))
         reportSyntaxError( "double attribute");
 
-    Pair(aMap,scope)
-  }
-
-  /** attribute value, terminated by either ' or ". value may not contain &lt;.
-   *       AttValue     ::= `'` { _  } `'`
-   *                      | `"` { _ } `"`
-   */
-  def xAttributeValue(): String = {
-    val endch = ch
-    nextch
-    while (ch != endch) {
-      if ('<' == ch)
-        reportSyntaxError( "'<' not allowed in attrib value" );
-      putChar(ch)
-      nextch
-    }
-    nextch
-    val str = cbuf.toString()
-    cbuf.setLength(0)
-
-    // well-formedness constraint
-    normalizeAttributeValue(str)
+    (aMap,scope)
   }
 
   /** entity value, terminated by either ' or ". value may not contain &lt;.
@@ -342,124 +326,54 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
   def xEntityValue(): String = {
     val endch = ch
     nextch
-    while (ch != endch) {
+    while (ch != endch && !eof) {
       putChar(ch)
       nextch
     }
     nextch
     val str = cbuf.toString()
-    cbuf.setLength(0)
+    cbuf.length = 0
     str
   }
 
-
-  /** parse a start or empty tag.
-   *  [40] STag         ::= '&lt;' Name { S Attribute } [S] 
-   *  [44] EmptyElemTag ::= '&lt;' Name { S Attribute } [S] 
-   */
-  protected def xTag(pscope:NamespaceBinding): Tuple3[String, MetaData, NamespaceBinding] = {
-    val qname = xName
-
-    xSpaceOpt
-    val Pair(aMap: MetaData, scope: NamespaceBinding) = {
-      if (isNameStart(ch)) 
-        xAttributes(pscope)
-      else 
-        Pair(Null, pscope)
-    }
-    Triple(qname, aMap, scope)
-  }
-
-  /** [42]  '&lt;' xmlEndTag ::=  '&lt;' '/' Name S? '&gt;'
-   */
-  def xEndTag(n: String) = {
-    xToken('/')
-    val m = xName
-    if (n != m)
-      reportSyntaxError("expected closing tag of " + n/* +", not "+m*/);
-    xSpaceOpt
-    xToken('>')
-  }
-
   /** '&lt;! CharData ::= [CDATA[ ( {char} - {char}"]]&gt;"{char} ) ']]&gt;'
    *
    * see [15]
    */
   def xCharData: NodeSeq = {
     xToken("[CDATA[")
-    val pos1 = pos
-    val sb: StringBuilder = new StringBuilder()
-    while (true) {
-      if (ch==']'  &&
-         { sb.append(ch); nextch; ch == ']' } &&
-         { sb.append(ch); nextch; ch == '>' } ) {
-        sb.setLength(sb.length() - 2);
-        nextch; 
-        return handle.text( pos1, sb.toString() );
-      } else sb.append( ch );
-      nextch; 
+    def mkResult(pos: Int, s: String): NodeSeq = {
+      handle.text(pos, s)
+      PCData(s)
     }
-    throw FatalError("this cannot happen");
-  };
-
-  /** CharRef ::= "&amp;#" '0'..'9' {'0'..'9'} ";"
-   *            | "&amp;#x" '0'..'9'|'A'..'F'|'a'..'f' { hexdigit } ";"
-   *
-   * see [66]
-   */
-  def xCharRef(ch: () => Char, nextch: () => Unit): String = {
-    Utility.parseCharRef(ch, nextch, &reportSyntaxError)
-    /*
-    val hex  = (ch() == 'x') && { nextch(); true };
-    val base = if (hex) 16 else 10;
-    var i = 0;
-    while (ch() != ';') {
-      ch() match {
-        case '0' | '1' | '2' | '3' | '4' | '5' | '6' | '7' | '8' | '9' =>
-          i = i * base + Character.digit( ch(), base );
-        case 'a' | 'b' | 'c' | 'd' | 'e' | 'f'
-           | 'A' | 'B' | 'C' | 'D' | 'E' | 'F' =>
-          if (! hex) 
-            reportSyntaxError("hex char not allowed in decimal char ref\n"
-                         +"Did you mean to write &#x ?");
-          else 
-            i = i * base + Character.digit(ch(), base);
-        case _ =>
-          reportSyntaxError("character '" + ch() + " not allowed in char ref\n");
-      }
-      nextch();
-    }
-    new String(Array(i.asInstanceOf[char]))
-    */
+    xTakeUntil(mkResult, () => pos, "]]>")
   }
 
-
   /** Comment ::= '&lt;!--' ((Char - '-') | ('-' (Char - '-')))* '--&gt;'
    *
    * see [15]
    */
   def xComment: NodeSeq = {
-    val sb: StringBuilder = new StringBuilder();
-    xToken('-');
-    xToken('-');
+    val sb: StringBuilder = new StringBuilder()
+    xToken("--")
     while (true) { 
       if (ch == '-'  && { sb.append(ch); nextch; ch == '-' }) {
-        sb.setLength(sb.length() - 1);
-        nextch; 
-        xToken('>');
-        return handle.comment(pos, sb.toString());
-      } else sb.append(ch);
-      nextch; 
+        sb.length = sb.length - 1
+        nextch
+        xToken('>')
+        return handle.comment(pos, sb.toString())
+      } else sb.append(ch)
+      nextch
     }
-    throw FatalError("this cannot happen");
-  };
+    throw FatalError("this cannot happen")
+  }
 
   /* todo: move this into the NodeBuilder class */
   def appendText(pos: Int, ts: NodeBuffer, txt: String): Unit = {
     if (preserveWS)
       ts &+ handle.text(pos, txt);
     else
-      for (val t <- TextBuffer.fromString(txt).toText) {
+      for (t <- TextBuffer.fromString(txt).toText) {
         ts &+ handle.text(pos, t.text);
       }
   }
@@ -486,69 +400,45 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
   def content(pscope: NamespaceBinding): NodeSeq = {
     var ts = new NodeBuffer
     var exit = eof
-    while (! exit) {
-      //Console.println("in content, ch = '"+ch+"' line="+scala.io.Position.line(pos));
-      /*      if( xEmbeddedBlock ) {
-       ts.append( xEmbeddedExpr );
-       } else {*/
-        tmppos = pos;
-        exit = eof;
-        if(!eof) 
-          ch match {
-          case '<' => // another tag
-            //Console.println("before ch = '"+ch+"' line="+scala.io.Position.line(pos)+" pos="+pos);
-            nextch; 
-            //Console.println("after ch = '"+ch+"' line="+scala.io.Position.line(pos)+" pos="+pos);
+    // todo: optimize seq repr.
+    def done = new NodeSeq { val theSeq = ts.toList }
 
-            if('/' ==ch)
-              exit = true;                    // end tag
-            else
-              content1(pscope, ts)
-          //case '{' => 
-/*            if( xCheckEmbeddedBlock ) {
-              ts.appendAll(xEmbeddedExpr);
-            } else {*/
-          //    val str = new StringBuilder("{");
-          //    str.append(xText);
-          //    appendText(tmppos, ts, str.toString());
-            /*}*/
-          // postcond: xEmbeddedBlock == false!
-          case '&' => // EntityRef or CharRef 
-            nextch;
-            ch match {
-              case '#' => // CharacterRef
-                nextch;
-                val theChar = handle.text( tmppos, 
-                                          xCharRef ({ ()=> ch },{ () => nextch }) );
-                xToken(';');
-                ts &+ theChar ;
-              case _ => // EntityRef
-                val n = xName
-                xToken(';')
-                n match {
-                  case "lt"    => ts &+ '<'
-                  case "gt"    => ts &+ '>'
-                  case "amp"   => ts &+ '&'
-                  case "quote" => ts &+ '"'
-                  case _ =>
-                    /*
-                     ts + handle.entityRef( tmppos, n ) ;
-                     */
-                    push(n)
-                }
-            }
-          case _ => // text content
-            //Console.println("text content?? pos = "+pos);
-            appendText(tmppos, ts, xText);
-          // here xEmbeddedBlock might be true
+    while (!exit) {
+      tmppos = pos
+      exit = eof
+      
+      if (eof)
+        return done
+      
+      ch match {
+        case '<' => // another tag
+          nextch match {
+            case '/'    => exit = true  // end tag
+            case _      => content1(pscope, ts)
           }
-    /*}*/
-    }
-    val list = ts.toList
-    // 2do: optimize seq repr.
-    new NodeSeq {
-      val theSeq = list
+
+        // postcond: xEmbeddedBlock == false!
+        case '&' => // EntityRef or CharRef 
+          nextch match {
+            case '#'  =>  // CharacterRef
+              nextch
+              val theChar = handle.text(tmppos, xCharRef(() => ch, () => nextch))
+              xToken(';');
+              ts &+ theChar
+            case _ =>     // EntityRef
+              val n = xName
+              xToken(';')
+            
+              if (unescape contains n) {
+                handle.entityRef(tmppos, n)
+                ts &+ unescape(n)
+              } else push(n)
+          }
+        case _ => // text content
+          appendText(tmppos, ts, xText);
+      }
     }
+    done
   } // content(NamespaceBinding)
 
   /** externalID ::= SYSTEM S syslit
@@ -598,47 +488,17 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
     if ((null != extID) && isValidating) {
 
       pushExternal(extID.systemId)
-      //val extSubsetSrc = externalSource( extID.systemId );
-
       extIndex = inpStack.length
-      /*
-       .indexOf(':') != -1) { // assume URI
-         Source.fromFile(new java.net.URI(extID.systemLiteral));
-       } else {
-         Source.fromFile(extID.systemLiteral);
-       }
-      */
-      //Console.println("I'll print it now");
-      //val old = curInput;
-      //tmppos = curInput.pos;
-      //val oldch = ch;
-      //curInput = extSubsetSrc;
-      //pos = 0;
-      //nextch;
 
       extSubset()
-
       pop()
-
       extIndex = -1
-
-      //curInput = old;
-      //pos = curInput.pos;
-      //ch = curInput.ch;
-      //eof = false;
-      //while(extSubsetSrc.hasNext)
-      //Console.print(extSubsetSrc.next);
-
-      //Console.println("returned from external, current ch = "+ch )
     }
 
     if ('[' == ch) { // internal subset
       nextch
       /* TODO */
-      //Console.println("hello");
       intSubset()
-      //while(']' != ch)
-      //  nextch;
       // TODO: do the DTD parsing?? ?!?!?!?!!
       xToken(']')
       xSpaceOpt
@@ -665,15 +525,14 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
    */
   def element1(pscope: NamespaceBinding): NodeSeq = {
     val pos = this.pos
-    val Tuple3(qname, aMap, scope) = xTag(pscope)
-    val Tuple2(pre, local) = Utility.prefix(qname) match {
-      case Some(p) => Pair(p,qname.substring(p.length()+1, qname.length()))
-      case _       => Pair(null,qname)
+    val (qname, (aMap, scope)) = xTag(pscope)
+    val (pre, local) = Utility.prefix(qname) match {
+      case Some(p) => (p, qname drop p.length+1)
+      case _       => (null, qname)
     }
     val ts = {
       if (ch == '/') {  // empty element
-        xToken('/')
-        xToken('>')
+        xToken("/>")
         handle.elemStart(pos, pre, local, aMap, scope)
         NodeSeq.Empty
       }
@@ -690,88 +549,21 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
     res
   }
 
-  //def xEmbeddedExpr: MarkupType;
-
-  /** Name ::= (Letter | '_' | ':') (NameChar)*
-   *
-   *  see  [5] of XML 1.0 specification
-   */
-  def xName: String = {
-    if (isNameStart(ch)) {
-      while (isNameChar(ch)) {
-        putChar(ch)
-        nextch
-      }
-      val n = cbuf.toString().intern()
-      cbuf.setLength(0)
-      n
-    } else {
-      reportSyntaxError("name expected")
-      new String()
-    }
-  }
-
-  /** scan [S] '=' [S]*/
-  def xEQ = { xSpaceOpt; xToken('='); xSpaceOpt }
-
-  /** skip optional space S? */
-  def xSpaceOpt = while (isSpace(ch) && !eof) { nextch; }
-
-  /** scan [3] S ::= (#x20 | #x9 | #xD | #xA)+ */
-  def xSpace =
-    if (isSpace(ch)) { nextch; xSpaceOpt }
-    else reportSyntaxError("whitespace expected")
-
-  /** '&lt;?' ProcInstr ::= Name [S ({Char} - ({Char}'&gt;?' {Char})]'?&gt;'
-   *
-   * see [15]
-   */
-  def xProcInstr: NodeSeq = {
-    val sb:StringBuilder = new StringBuilder()
-    val n = xName
-    if (isSpace(ch)) {
-      xSpace
-      while (true) {
-        if (ch == '?' && { sb.append( ch ); nextch; ch == '>' }) {
-          sb.setLength(sb.length() - 1);
-          nextch;
-          return handle.procInstr(tmppos, n.toString(), sb.toString());
-        } else
-          sb.append(ch);
-        nextch
-      }
-    };
-    xToken('?')
-    xToken('>')
-    handle.procInstr(tmppos, n.toString(), sb.toString())
-  }
-
   /** parse character data.
    *   precondition: xEmbeddedBlock == false (we are not in a scala block)
    */
   def xText: String = {
-    //if( xEmbeddedBlock ) throw FatalError("internal error: encountered embedded block"); // assert
+    var exit = false;
+    while (! exit) {
+      putChar(ch);
+      val opos = pos;
+      nextch;
 
-    /*if( xCheckEmbeddedBlock )
-      return ""
-    else {*/
-    //Console.println("in xText! ch = '"+ch+"'");
-      var exit = false;
-      while (! exit) {
-        //Console.println("LOOP in xText! ch = '"+ch+"' + pos="+pos);
-        putChar(ch);
-        val opos = pos;
-        nextch;
-
-        //Console.println("STILL LOOP in xText! ch = '"+ch+"' + pos="+pos+" opos="+opos);
-        
-
-        exit = eof || /*{ nextch; xCheckEmbeddedBlock }||*/( ch == '<' ) || ( ch == '&' );
-      }
-      val str = cbuf.toString();
-      cbuf.setLength(0);
-      str
-    /*}*/
+      exit = eof || ( ch == '<' ) || ( ch == '&' )
+    }
+    val str = cbuf.toString();
+    cbuf.length = 0;
+    str
   }
 
   /** attribute value, terminated by either ' or ". value may not contain &lt;.
@@ -783,24 +575,23 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
     if (ch != '\'' && ch != '"')
       reportSyntaxError("quote ' or \" expected");
     nextch
-    while (ch != endch) {
+    while (ch != endch && !eof) {
       putChar(ch)
       nextch
     }
     nextch
     val str = cbuf.toString()
-    cbuf.setLength(0)
+    cbuf.length = 0
     str
   }
 
-
   /* [12]       PubidLiteral ::=        '"' PubidChar* '"' | "'" (PubidChar - "'")* "'" */
   def pubidLiteral(): String = {
     val endch = ch
     if (ch!='\'' && ch != '"')
       reportSyntaxError("quote ' or \" expected");
     nextch
-    while (ch != endch) {
+    while (ch != endch && !eof) {
       putChar(ch)
       //Console.println("hello '"+ch+"'"+isPubIDChar(ch));
       if (!isPubIDChar(ch))
@@ -809,7 +600,7 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
     }
     nextch
     val str = cbuf.toString()
-    cbuf.setLength(0)
+    cbuf.length = 0
     str
   }
 
@@ -872,34 +663,16 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
               val ent = xName
               xToken(';')
               xSpaceOpt
-            /*
-              Console.println("hello, pushing!");
-            {
-              val test =  replacementText(ent);
-              while(test.hasNext)
-                Console.print(test.next);
-            } */
+
               push(ent)
               xSpaceOpt
-              //Console.println("hello, getting name");
               val stmt = xName
-              //Console.println("hello, got name");
               xSpaceOpt
-            //Console.println("how can we be eof = "+eof);
 
-            // eof = true because not external?!
-              //if(!eof)
-              //  error("expected only INCLUDE or IGNORE");
-
-              //pop();
-
-              //Console.println("hello, popped");
               stmt match {
                 // parameter entity
-                case "INCLUDE" =>
-                  doInclude()
-                case "IGNORE" =>
-                  doIgnore()
+                case "INCLUDE"  => doInclude()
+                case "IGNORE"   => doIgnore()
               }
             case 'I' =>
               nextch
@@ -943,14 +716,13 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
     case _ if isSpace(ch) =>
       xSpace
     case _ =>
-      reportSyntaxError("markupdecl: unexpected character '"+ch+"' #" + ch.asInstanceOf[Int])
+      reportSyntaxError("markupdecl: unexpected character '"+ch+"' #" + ch.toInt)
       nextch
   }
 
-  /**  "rec-xml/#ExtSubset" pe references may not occur within markup 
-   declarations 
+  /**  "rec-xml/#ExtSubset" pe references may not occur within markup declarations 
    */
-  def intSubset(): Unit = {
+  def intSubset() {
     //Console.println("(DEBUG) intSubset()")
     xSpace
     while (']' != ch)
@@ -959,7 +731,7 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
 
   /** &lt;! element := ELEMENT
    */
-  def elementDecl(): Unit = {
+  def elementDecl() {
     xToken("EMENT")
     xSpace
     val n = xName
@@ -972,7 +744,7 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
     //Console.println("END["+ch+"]")
     nextch
     val cmstr = cbuf.toString()
-    cbuf.setLength(0)
+    cbuf.length = 0
     handle.elemDecl(n, cmstr)
   }
 
@@ -984,11 +756,10 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
     val n = xName
     xSpace
     var attList: List[AttrDecl] = Nil
+    
     // later: find the elemDecl for n
     while ('>' != ch) {
       val aname = xName
-      //Console.println("attribute name: "+aname);
-      var defdecl: DefaultDecl = null
       xSpace
       // could be enumeration (foo,bar) parse this later :-/
       while ('"' != ch && '\'' != ch && '#' != ch && '<' != ch) {
@@ -997,31 +768,26 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
         nextch;
       }
       val atpe = cbuf.toString()
-      cbuf.setLength(0)
-      //Console.println("attr type: "+atpe);
-      ch match {
+      cbuf.length = 0
+      
+      val defdecl: DefaultDecl = ch match {
         case '\'' | '"' =>
-          val defValue = xAttributeValue() // default value
-          defdecl = DEFAULT(false, defValue)
+          DEFAULT(false, xAttributeValue())
 
         case '#' =>
           nextch
           xName match {
-            case "FIXED" =>
-              xSpace
-              val defValue = xAttributeValue() // default value
-              defdecl = DEFAULT(true, defValue)
-            case "IMPLIED" =>
-              defdecl = IMPLIED
-            case "REQUIRED" =>
-              defdecl = REQUIRED
+            case "FIXED"    => xSpace ; DEFAULT(true, xAttributeValue())
+            case "IMPLIED"  => IMPLIED
+            case "REQUIRED" => REQUIRED
           }
         case _ =>
+          null
       }
       xSpaceOpt
 
-      attList = AttrDecl(aname, atpe, defdecl) :: attList
-      cbuf.setLength(0)
+      attList ::= AttrDecl(aname, atpe, defdecl)
+      cbuf.length = 0
     }
     nextch
     handle.attListDecl(n, attList.reverse)
@@ -1078,7 +844,7 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
 
   /** 'N' notationDecl ::= "OTATION"
    */
-  def notationDecl(): Unit = {
+  def notationDecl() {
     xToken("OTATION")
     xSpace
     val notat = xName
@@ -1100,33 +866,18 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
       new PublicID(pubID, sysID);
     } else {
       reportSyntaxError("PUBLIC or SYSTEM expected");
-      error("died parsing notationdecl")
+      sys.error("died parsing notationdecl")
     }
     xSpaceOpt
     xToken('>')
     handle.notationDecl(notat, extID)
   }
 
-  /**
-   * report a syntax error
-   */
-  def reportSyntaxError(pos: int, str: String): Unit = {
-    curInput.reportError(pos, str)
-    //error("MarkupParser::synerr") // DEBUG
-  }
-
+  def reportSyntaxError(pos: Int, str: String): Unit = curInput.reportError(pos, str)
   def reportSyntaxError(str: String): Unit = reportSyntaxError(pos, str)
+  def reportValidationError(pos: Int, str: String): Unit = reportSyntaxError(pos, str)
 
-  /**
-   * report a syntax error
-   */
-  def reportValidationError(pos: int, str: String): Unit =
-    curInput.reportError(pos, str)
-
-  def push(entityName:String) = {
-    //Console.println("BEFORE PUSHING  "+ch)
-    //Console.println("BEFORE PUSHING  "+pos)
-    //Console.print("[PUSHING "+entityName+"]")
+  def push(entityName: String) {
     if (!eof)
       inpStack = curInput :: inpStack
 
@@ -1134,80 +885,19 @@ trait MarkupParser requires (MarkupParser with MarkupHandler) extends AnyRef wit
     nextch
   }
 
-  /*
-  def push(src:Source) = {
-    curInput = src
-    nextch
-  }
-  */
-
-  def pushExternal(systemId:String) = {
-    //Console.print("BEFORE PUSH, curInput = $"+curInput.descr)
-    //Console.println(" stack = "+inpStack.map { x => "$"+x.descr })
-
-    //Console.print("[PUSHING EXTERNAL "+systemId+"]")
+  def pushExternal(systemId: String) {
     if (!eof)
       inpStack = curInput :: inpStack
 
     curInput = externalSource(systemId)
-
-    //Console.print("AFTER PUSH, curInput = $"+curInput.descr)
-    //Console.println(" stack = "+inpStack.map { x => "$"+x.descr })
-
     nextch
   }
 
-  def pop() = {
+  def pop() {
     curInput = inpStack.head
     inpStack = inpStack.tail
     ch = curInput.ch
     pos = curInput.pos
     eof = false // must be false, because of places where entity refs occur
-    //Console.println("\n AFTER POP, curInput = $"+curInput.descr);
-    //Console.println(inpStack.map { x => x.descr });
   }
-
-  /** for the moment, replace only character references 
-   *  see spec 3.3.3
-   *  precond: cbuf empty
-   */
-  def normalizeAttributeValue(attval: String) = {
-    val s: Seq[Char] = attval
-    val it = s.elements
-    while(it.hasNext) {
-      it.next match {
-        case ' '|'\t'|'\n'|'\r' => 
-          cbuf.append(' ');
-        case '&' => it.next match {
-          case '#' =>
-            var c = it.next
-            val s = xCharRef ({ () => c }, { () => c = it.next })
-            cbuf.append(s)
-          case nchar =>
-            val nbuf = new StringBuilder()
-            var d = nchar
-            do {
-              nbuf.append(d)
-              d = it.next
-            } while(d != ';');
-            nbuf.toString() match {
-              case "lt"    => cbuf.append('<')
-              case "gt"    => cbuf.append('>')
-              case "amp"   => cbuf.append('&')
-              case "quote" => cbuf.append('"')
-              case name =>
-                //don't handle entityrefs for now
-                cbuf.append('&')
-                cbuf.append(name)
-                cbuf.append(';')
-            }
-        }
-        case c => cbuf.append(c)
-      }
-    }
-    val name = cbuf.toString()
-    cbuf.setLength(0)
-    name
-  }
-
 }
