@@ -1,14 +1,15 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2002-2006, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2002-2011, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: RewriteRule.scala 8624 2006-09-08 09:33:07Z michelou $
 
-package scala.xml.transform
+
+package scala.xml
+package transform
 
 /** a RewriteRule, when applied to a term, yields either
  *  the resulting of rewriting or the term itself it the rule
